@@ -24,6 +24,10 @@
 #include "skia/include/core/SkRegion.h"
 #include "skia/include/effects/SkImageFilters.h"
 #include "skia/include/private/base/SkMalloc.h"
+#include "skia/src/core/SkEffectPriv.h"
+#include "skia/src/core/SkRasterPipeline.h"
+#include "skia/src/core/SkWriteBuffer.h"
+#include "skia/src/shaders/SkEmptyShader.h"
 #include "Blur.h"
 #include "Logging.h"
 #include "Tools.h"
@@ -257,7 +261,7 @@ static sk_sp<SkImage> GetSkImageForSurface(SourceSurface* aSurface,
   }
 
   DataSourceSurface::MappedSurface map;
-  SkImage::RasterReleaseProc releaseProc;
+  void (*releaseProc)(const void*, void*);
   if (dataSurface->GetType() == SurfaceType::DATA_SHARED_WRAPPER) {
     // Technically all surfaces should be mapped and unmapped explicitly but it
     // appears SourceSurfaceSkia and DataSourceSurfaceWrapper have issues with
@@ -283,7 +287,7 @@ static sk_sp<SkImage> GetSkImageForSurface(SourceSurface* aSurface,
 
   SkPixmap pixmap(MakeSkiaImageInfo(surf->GetSize(), surf->GetFormat()),
                   map.mData, map.mStride);
-  sk_sp<SkImage> image = SkImage::MakeFromRaster(pixmap, releaseProc, surf);
+  sk_sp<SkImage> image = SkImages::RasterFromPixmap(pixmap, releaseProc, surf);
   if (!image) {
     releaseProc(map.mData, surf);
     gfxDebug() << "Failed making Skia raster image for temporary surface";
@@ -347,7 +351,7 @@ already_AddRefed<SourceSurface> DrawTargetSkia::Snapshot(
     // pixels.
     SkPixmap pixmap;
     if (mSurface->peekPixels(&pixmap)) {
-      image = SkImage::MakeFromRaster(pixmap, nullptr, nullptr);
+      image = SkImages::RasterFromPixmap(pixmap, nullptr, nullptr);
     } else {
       image = mSurface->makeImageSnapshot();
     }
@@ -412,10 +416,10 @@ static sk_sp<SkImage> ExtractSubset(sk_sp<SkImage> aImage,
       pixmap.extractSubset(&subsetPixmap, subsetRect)) {
     // Release the original image reference so only the subset image keeps it
     // alive.
-    return SkImage::MakeFromRaster(subsetPixmap, ReleaseImage,
-                                   aImage.release());
+    return SkImages::RasterFromPixmap(subsetPixmap, ReleaseImage,
+                                      aImage.release());
   }
-  return aImage->makeSubset(subsetRect);
+  return aImage->makeSubset(nullptr, subsetRect);
 }
 
 static void FreeAlphaPixels(void* aBuf, void*) { sk_free(aBuf); }
@@ -584,15 +588,19 @@ static void SetPaintPattern(SkPaint& aPaint, const Pattern& aPattern,
     }
     case PatternType::SURFACE: {
       const SurfacePattern& pat = static_cast<const SurfacePattern&>(aPattern);
+      Matrix offsetMatrix = pat.mMatrix;
+      if (pat.mSurface) {
+        offsetMatrix.PreTranslate(pat.mSurface->GetRect().TopLeft());
+      }
       sk_sp<SkImage> image =
-          GetSkImageForSurface(pat.mSurface, &aLock, aBounds, &pat.mMatrix);
+          GetSkImageForSurface(pat.mSurface, &aLock, aBounds, &offsetMatrix);
       if (!image) {
         aPaint.setColor(SK_ColorTRANSPARENT);
         break;
       }
 
       SkMatrix mat;
-      GfxMatrixToSkiaMatrix(pat.mMatrix, mat);
+      GfxMatrixToSkiaMatrix(offsetMatrix, mat);
       if (aMatrix) {
         mat.postConcat(*aMatrix);
       }
@@ -711,7 +719,7 @@ struct AutoPaintSetup {
   Float mAlpha;
 };
 
-void DrawTargetSkia::Flush() { mCanvas->flush(); }
+void DrawTargetSkia::Flush() {}
 
 void DrawTargetSkia::DrawSurface(SourceSurface* aSurface, const Rect& aDest,
                                  const Rect& aSource,
@@ -851,10 +859,12 @@ void DrawTargetSkia::FillRect(const Rect& aRect, const Pattern& aPattern,
              kOpaque_SkAlphaType) &&
         !pat.mMatrix.HasNonAxisAlignedTransform()) {
       // Bound the sampling to smaller of the bounds or the sampling rect.
-      IntRect srcRect(IntPoint(0, 0), pat.mSurface->GetSize());
+      IntRect surfaceBounds = pat.mSurface->GetRect();
+      IntRect srcRect(IntPoint(0, 0), surfaceBounds.Size());
       if (!pat.mSamplingRect.IsEmpty()) {
         srcRect = srcRect.Intersect(pat.mSamplingRect);
       }
+      srcRect.MoveBy(surfaceBounds.TopLeft());
       // Transform the destination rectangle by the inverse of the pattern
       // matrix so that it is in pattern space like the source rectangle.
       Rect patRect = aRect - pat.mMatrix.GetTranslation();
@@ -1252,7 +1262,8 @@ static bool CanDrawFont(ScaledFont* aFont) {
 void DrawTargetSkia::DrawGlyphs(ScaledFont* aFont, const GlyphBuffer& aBuffer,
                                 const Pattern& aPattern,
                                 const StrokeOptions* aStrokeOptions,
-                                const DrawOptions& aOptions) {
+                                const DrawOptions& aOptions,
+                                SkShader* aShader) {
   if (!CanDrawFont(aFont)) {
     return;
   }
@@ -1288,6 +1299,10 @@ void DrawTargetSkia::DrawGlyphs(ScaledFont* aFont, const GlyphBuffer& aBuffer,
 
   skiaFont->SetupSkFontDrawOptions(font);
 
+  if (aShader) {
+    paint.mPaint.setShader(sk_ref_sp(aShader));
+  }
+
   // Limit the amount of internal batch allocations Skia does.
   const uint32_t kMaxGlyphBatchSize = 8192;
 
@@ -1306,6 +1321,51 @@ void DrawTargetSkia::DrawGlyphs(ScaledFont* aFont, const GlyphBuffer& aBuffer,
   }
 }
 
+// This shader overrides the luminance color used to generate the preblend
+// tables for glyphs, without actually changing the rasterized color. This is
+// necesary for subpixel AA blending which requires both the mask and color
+// as separate inputs.
+class GlyphMaskShader : public SkEmptyShader {
+ public:
+  explicit GlyphMaskShader(const DeviceColor& aColor)
+      : mColor({aColor.r, aColor.g, aColor.b, aColor.a}) {}
+
+  bool onAsLuminanceColor(SkColor4f* aLum) const override {
+    *aLum = mColor;
+    return true;
+  }
+
+  bool isOpaque() const override { return true; }
+  bool isConstant() const override { return true; }
+
+  void flatten(SkWriteBuffer& buffer) const override {
+    buffer.writeColor4f(mColor);
+  }
+
+  bool appendStages(const SkStageRec& rec,
+                    const SkShaders::MatrixRec&) const override {
+    rec.fPipeline->appendConstantColor(rec.fAlloc,
+                                       SkColor4f{1, 1, 1, 1}.premul().vec());
+    return true;
+  }
+
+ private:
+  SkColor4f mColor;
+};
+
+void DrawTargetSkia::DrawGlyphMask(ScaledFont* aFont,
+                                   const GlyphBuffer& aBuffer,
+                                   const DeviceColor& aColor,
+                                   const StrokeOptions* aStrokeOptions,
+                                   const DrawOptions& aOptions) {
+  // Draw a mask using the GlyphMaskShader that can be used for subpixel AA
+  // but that uses the gamma preblend weighting of the given color, even though
+  // the mask itself does not use that color.
+  sk_sp<GlyphMaskShader> shader = sk_make_sp<GlyphMaskShader>(aColor);
+  DrawGlyphs(aFont, aBuffer, ColorPattern(DeviceColor(1, 1, 1, 1)),
+             aStrokeOptions, aOptions, shader.get());
+}
+
 Maybe<Rect> DrawTargetSkia::GetGlyphLocalBounds(
     ScaledFont* aFont, const GlyphBuffer& aBuffer, const Pattern& aPattern,
     const StrokeOptions* aStrokeOptions, const DrawOptions& aOptions) {
@@ -1376,6 +1436,8 @@ Maybe<Rect> DrawTargetSkia::GetGlyphLocalBounds(
     return Nothing();
   }
 
+  // Inflate the bounds to account for potential font hinting.
+  bounds.Inflate(1);
   return Some(bounds);
 }
 
@@ -1398,7 +1460,7 @@ void DrawTargetSkia::Mask(const Pattern& aSource, const Pattern& aMask,
   SkPaint maskPaint;
   SetPaintPattern(maskPaint, aMask, lock);
 
-  sk_sp<SkShader> maskShader(maskPaint.getShader());
+  sk_sp<SkShader> maskShader(maskPaint.refShader());
   if (!maskShader && maskPaint.getAlpha() != 0xFF) {
     if (maskPaint.getAlpha() == 0) {
       return;
@@ -1486,6 +1548,9 @@ bool DrawTarget::Draw3DTransformedSurface(SourceSurface* aSurface,
   }
 
   DataSourceSurface::ScopedMap map(dstSurf, DataSourceSurface::READ_WRITE);
+  if (!map.IsMapped()) {
+    return false;
+  }
   std::unique_ptr<SkCanvas> dstCanvas(SkCanvas::MakeRasterDirect(
       SkImageInfo::Make(xformBounds.Width(), xformBounds.Height(),
                         GfxFormatToSkiaColorType(dstSurf->GetFormat()),
@@ -1506,7 +1571,6 @@ bool DrawTarget::Draw3DTransformedSurface(SourceSurface* aSurface,
 
   dstCanvas->drawImage(srcImage, 0, 0, SkSamplingOptions(SkFilterMode::kLinear),
                        &paint);
-  dstCanvas->flush();
 
   // Temporarily reset the DT's transform, since it has already been composed
   // above.
@@ -1630,16 +1694,21 @@ DrawTargetSkia::OptimizeSourceSurfaceForUnknownAlpha(
     return surface.forget();
   }
 
-  RefPtr<DataSourceSurface> dataSurface = aSurface->GetDataSurface();
-  DataSourceSurface::ScopedMap map(dataSurface, DataSourceSurface::READ_WRITE);
+  if (RefPtr<DataSourceSurface> dataSurface = aSurface->GetDataSurface()) {
+    DataSourceSurface::ScopedMap map(dataSurface,
+                                     DataSourceSurface::READ_WRITE);
+    if (map.IsMapped()) {
+      // For plugins, GDI can sometimes just write 0 to the alpha channel
+      // even for RGBX formats. In this case, we have to manually write
+      // the alpha channel to make Skia happy with RGBX and in case GDI
+      // writes some bad data. Luckily, this only happens on plugins.
+      WriteRGBXFormat(map.GetData(), dataSurface->GetSize(), map.GetStride(),
+                      dataSurface->GetFormat());
+      return dataSurface.forget();
+    }
+  }
 
-  // For plugins, GDI can sometimes just write 0 to the alpha channel
-  // even for RGBX formats. In this case, we have to manually write
-  // the alpha channel to make Skia happy with RGBX and in case GDI
-  // writes some bad data. Luckily, this only happens on plugins.
-  WriteRGBXFormat(map.GetData(), dataSurface->GetSize(), map.GetStride(),
-                  dataSurface->GetFormat());
-  return dataSurface.forget();
+  return nullptr;
 }
 
 already_AddRefed<SourceSurface> DrawTargetSkia::OptimizeSourceSurface(
@@ -1653,13 +1722,18 @@ already_AddRefed<SourceSurface> DrawTargetSkia::OptimizeSourceSurface(
   // uploading, so any data surface is fine. Call GetDataSurface
   // to trigger any required readback so that it only happens
   // once.
-  RefPtr<DataSourceSurface> dataSurface = aSurface->GetDataSurface();
+  if (RefPtr<DataSourceSurface> dataSurface = aSurface->GetDataSurface()) {
 #ifdef DEBUG
-  DataSourceSurface::ScopedMap map(dataSurface, DataSourceSurface::READ);
-  MOZ_ASSERT(VerifyRGBXFormat(map.GetData(), dataSurface->GetSize(),
-                              map.GetStride(), dataSurface->GetFormat()));
+    DataSourceSurface::ScopedMap map(dataSurface, DataSourceSurface::READ);
+    if (map.IsMapped()) {
+      MOZ_ASSERT(VerifyRGBXFormat(map.GetData(), dataSurface->GetSize(),
+                                  map.GetStride(), dataSurface->GetFormat()));
+    }
 #endif
-  return dataSurface.forget();
+    return dataSurface.forget();
+  }
+
+  return nullptr;
 }
 
 already_AddRefed<SourceSurface>
@@ -1685,10 +1759,12 @@ void DrawTargetSkia::CopySurface(SourceSurface* aSurface,
   }
 
   // Ensure the source rect intersects the surface bounds.
-  IntRect srcRect = aSourceRect.Intersect(SkIRectToIntRect(srcPixmap.bounds()));
+  IntRect offsetSrcRect = aSourceRect - aSurface->GetRect().TopLeft();
+  IntRect srcRect =
+      offsetSrcRect.Intersect(SkIRectToIntRect(srcPixmap.bounds()));
   // Move the destination offset to match the altered source rect.
   IntPoint dstOffset =
-      aDestination + (srcRect.TopLeft() - aSourceRect.TopLeft());
+      aDestination + (srcRect.TopLeft() - offsetSrcRect.TopLeft());
   // Then ensure the dest rect intersect the canvas bounds.
   IntRect dstRect = IntRect(dstOffset, srcRect.Size()).Intersect(GetRect());
   // Move the source rect to match the altered dest rect.
@@ -1721,8 +1797,11 @@ bool DrawTargetSkia::Init(const IntSize& aSize, SurfaceFormat aFormat) {
   // we need to have surfaces that have a stride aligned to 4 for interop with
   // cairo
   SkImageInfo info = MakeSkiaImageInfo(aSize, aFormat);
+  if (info.bytesPerPixel() != BytesPerPixel(aFormat)) {
+    return false;
+  }
   size_t stride = GetAlignedStride<4>(info.width(), info.bytesPerPixel());
-  if (!stride) {
+  if (!stride || stride < info.minRowBytes64()) {
     return false;
   }
   SkSurfaceProps props(0, GetSkPixelGeometry());
@@ -1743,10 +1822,10 @@ bool DrawTargetSkia::Init(const IntSize& aSize, SurfaceFormat aFormat) {
     if (!buf) {
       return false;
     }
-    mSurface = AsRefPtr(SkSurface::MakeRasterDirectReleaseProc(
+    mSurface = AsRefPtr(SkSurfaces::WrapPixels(
         info, buf, stride, FreeAlphaPixels, nullptr, &props));
   } else {
-    mSurface = AsRefPtr(SkSurface::MakeRaster(info, stride, &props));
+    mSurface = AsRefPtr(SkSurfaces::Raster(info, stride, &props));
   }
   if (!mSurface) {
     return false;
@@ -1791,9 +1870,14 @@ bool DrawTargetSkia::Init(unsigned char* aData, const IntSize& aSize,
   MOZ_ASSERT((aFormat != SurfaceFormat::B8G8R8X8) || aUninitialized ||
              VerifyRGBXFormat(aData, aSize, aStride, aFormat));
 
+  SkImageInfo info = MakeSkiaImageInfo(aSize, aFormat);
+  if (info.bytesPerPixel() != BytesPerPixel(aFormat) || aStride <= 0 ||
+      size_t(aStride) < info.minRowBytes64()) {
+    return false;
+  }
+
   SkSurfaceProps props(0, GetSkPixelGeometry());
-  mSurface = AsRefPtr(SkSurface::MakeRasterDirect(
-      MakeSkiaImageInfo(aSize, aFormat), aData, aStride, &props));
+  mSurface = AsRefPtr(SkSurfaces::WrapPixels(info, aData, aStride, &props));
   if (!mSurface) {
     return false;
   }
@@ -1818,8 +1902,15 @@ bool DrawTargetSkia::Init(RefPtr<DataSourceSurface>&& aSurface) {
   MOZ_ASSERT((format != SurfaceFormat::B8G8R8X8) ||
              VerifyRGBXFormat(map->GetData(), size, map->GetStride(), format));
 
+  SkImageInfo info = MakeSkiaImageInfo(size, format);
+  if (info.bytesPerPixel() != BytesPerPixel(format) ||
+      size_t(map->GetStride()) < info.minRowBytes64()) {
+    delete map;
+    return false;
+  }
+
   SkSurfaceProps props(0, GetSkPixelGeometry());
-  mSurface = AsRefPtr(SkSurface::MakeRasterDirectReleaseProc(
+  mSurface = AsRefPtr(SkSurfaces::WrapPixels(
       MakeSkiaImageInfo(size, format), map->GetData(), map->GetStride(),
       DrawTargetSkia::ReleaseMappedSkSurface, map, &props));
   if (!mSurface) {
@@ -1860,14 +1951,12 @@ already_AddRefed<PathBuilder> DrawTargetSkia::CreatePathBuilder(
 
 void DrawTargetSkia::ClearRect(const Rect& aRect) {
   MarkChanged();
-  mCanvas->save();
-  // Restrict clearing to the clip region if requested
-  mCanvas->clipRect(RectToSkRect(aRect), SkClipOp::kIntersect, true);
-  SkColor clearColor = (mFormat == SurfaceFormat::B8G8R8X8)
-                           ? SK_ColorBLACK
-                           : SK_ColorTRANSPARENT;
-  mCanvas->clear(clearColor);
-  mCanvas->restore();
+  SkPaint paint;
+  paint.setAntiAlias(true);
+  paint.setColor((mFormat == SurfaceFormat::B8G8R8X8) ? SK_ColorBLACK
+                                                      : SK_ColorTRANSPARENT);
+  paint.setBlendMode(SkBlendMode::kSrc);
+  mCanvas->drawRect(RectToSkRect(aRect), paint);
 }
 
 void DrawTargetSkia::PushClip(const Path* aPath) {
