@@ -103,15 +103,15 @@
 #include "RootCertificateTelemetryUtils.h"
 #include "ScopedNSSTypes.h"
 #include "SharedCertVerifier.h"
-#include "SharedSSLState.h"
 #include "VerifySSLServerCertChild.h"
 #include "cert.h"
 #include "mozilla/Assertions.h"
 #include "mozilla/Casting.h"
 #include "mozilla/RefPtr.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/StaticPrefs_security.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/Unused.h"
+#include "mozilla/glean/SecurityManagerSslMetrics.h"
 #include "nsComponentManagerUtils.h"
 #include "nsContentUtils.h"
 #include "nsICertOverrideService.h"
@@ -162,9 +162,10 @@ void InitializeSSLServerCertVerificationThreads() {
   gCertVerificationThreadPool = new nsThreadPool();
   NS_ADDREF(gCertVerificationThreadPool);
 
-  (void)gCertVerificationThreadPool->SetIdleThreadLimit(5);
-  (void)gCertVerificationThreadPool->SetIdleThreadTimeout(30 * 1000);
   (void)gCertVerificationThreadPool->SetThreadLimit(5);
+  (void)gCertVerificationThreadPool->SetIdleThreadLimit(1);
+  (void)gCertVerificationThreadPool->SetIdleThreadMaximumTimeout(30 * 1000);
+  (void)gCertVerificationThreadPool->SetIdleThreadGraceTimeout(500);
   (void)gCertVerificationThreadPool->SetName("SSL Cert"_ns);
 }
 
@@ -226,6 +227,9 @@ uint32_t MapOverridableErrorToProbeValue(PRErrorCode errorCode) {
       return 19;
     case mozilla::pkix::MOZILLA_PKIX_ERROR_MITM_DETECTED:
       return 20;
+    case mozilla::pkix::
+        MOZILLA_PKIX_ERROR_INSUFFICIENT_CERTIFICATE_TRANSPARENCY:
+      return 21;
   }
   NS_WARNING(
       "Unknown certificate error code. Does MapOverridableErrorToProbeValue "
@@ -269,14 +273,16 @@ static uint32_t MapCertErrorToProbeValue(PRErrorCode errorCode) {
 Maybe<nsITransportSecurityInfo::OverridableErrorCategory>
 CategorizeCertificateError(PRErrorCode certificateError) {
   switch (certificateError) {
+    case SEC_ERROR_CA_CERT_INVALID:
     case SEC_ERROR_CERT_SIGNATURE_ALGORITHM_DISABLED:
     case SEC_ERROR_EXPIRED_ISSUER_CERTIFICATE:
     case SEC_ERROR_UNKNOWN_ISSUER:
-    case SEC_ERROR_CA_CERT_INVALID:
     case mozilla::pkix::MOZILLA_PKIX_ERROR_ADDITIONAL_POLICY_CONSTRAINT_FAILED:
     case mozilla::pkix::MOZILLA_PKIX_ERROR_CA_CERT_USED_AS_END_ENTITY:
     case mozilla::pkix::MOZILLA_PKIX_ERROR_EMPTY_ISSUER_NAME:
     case mozilla::pkix::MOZILLA_PKIX_ERROR_INADEQUATE_KEY_SIZE:
+    case mozilla::pkix::
+        MOZILLA_PKIX_ERROR_INSUFFICIENT_CERTIFICATE_TRANSPARENCY:
     case mozilla::pkix::MOZILLA_PKIX_ERROR_MITM_DETECTED:
     case mozilla::pkix::MOZILLA_PKIX_ERROR_NOT_YET_VALID_ISSUER_CERTIFICATE:
     case mozilla::pkix::MOZILLA_PKIX_ERROR_SELF_SIGNED_CERT:
@@ -288,8 +294,8 @@ CategorizeCertificateError(PRErrorCode certificateError) {
       return Some(
           nsITransportSecurityInfo::OverridableErrorCategory::ERROR_DOMAIN);
 
-    case SEC_ERROR_INVALID_TIME:
     case SEC_ERROR_EXPIRED_CERTIFICATE:
+    case SEC_ERROR_INVALID_TIME:
     case mozilla::pkix::MOZILLA_PKIX_ERROR_NOT_YET_VALID_CERTIFICATE:
       return Some(
           nsITransportSecurityInfo::OverridableErrorCategory::ERROR_TIME);
@@ -436,50 +442,22 @@ static SECStatus BlockServerCertChangeForSpdy(
 }
 
 void GatherTelemetryForSingleSCT(const ct::VerifiedSCT& verifiedSct) {
-  // See SSL_SCTS_ORIGIN in Histograms.json.
-  uint32_t origin = 0;
-  switch (verifiedSct.origin) {
-    case ct::VerifiedSCT::Origin::Embedded:
-      origin = 1;
-      break;
-    case ct::VerifiedSCT::Origin::TLSExtension:
-      origin = 2;
-      break;
-    case ct::VerifiedSCT::Origin::OCSPResponse:
-      origin = 3;
-      break;
-    default:
-      MOZ_ASSERT_UNREACHABLE("Unexpected VerifiedSCT::Origin type");
-  }
-  Telemetry::Accumulate(Telemetry::SSL_SCTS_ORIGIN, origin);
-
-  // See SSL_SCTS_VERIFICATION_STATUS in Histograms.json.
+  // See scts_verification_status in metrics.yaml.
   uint32_t verificationStatus = 0;
-  switch (verifiedSct.status) {
-    case ct::VerifiedSCT::Status::Valid:
+  switch (verifiedSct.logState) {
+    case ct::CTLogState::Admissible:
       verificationStatus = 1;
       break;
-    case ct::VerifiedSCT::Status::UnknownLog:
-      verificationStatus = 2;
-      break;
-    case ct::VerifiedSCT::Status::InvalidSignature:
-      verificationStatus = 3;
-      break;
-    case ct::VerifiedSCT::Status::InvalidTimestamp:
-      verificationStatus = 4;
-      break;
-    case ct::VerifiedSCT::Status::ValidFromDisqualifiedLog:
+    case ct::CTLogState::Retired:
       verificationStatus = 5;
       break;
-    default:
-      MOZ_ASSERT_UNREACHABLE("Unexpected VerifiedSCT::Status type");
   }
-  Telemetry::Accumulate(Telemetry::SSL_SCTS_VERIFICATION_STATUS,
-                        verificationStatus);
+  glean::ssl::scts_verification_status.AccumulateSingleSample(
+      verificationStatus);
 }
 
 void GatherCertificateTransparencyTelemetry(
-    const nsTArray<uint8_t>& rootCert, bool isEV,
+    const nsTArray<uint8_t>& rootCert,
     const CertificateTransparencyInfo& info) {
   if (!info.enabled) {
     // No telemetry is gathered when CT is disabled.
@@ -490,10 +468,29 @@ void GatherCertificateTransparencyTelemetry(
     GatherTelemetryForSingleSCT(sct);
   }
 
-  // Decoding errors are reported to the 0th bucket
-  // of the SSL_SCTS_VERIFICATION_STATUS enumerated probe.
+  // See scts_verification_status in metrics.yaml.
   for (size_t i = 0; i < info.verifyResult.decodingErrors; ++i) {
-    Telemetry::Accumulate(Telemetry::SSL_SCTS_VERIFICATION_STATUS, 0);
+    glean::ssl::scts_verification_status.AccumulateSingleSample(0);
+  }
+  for (size_t i = 0; i < info.verifyResult.sctsFromUnknownLogs; ++i) {
+    glean::ssl::scts_verification_status.AccumulateSingleSample(2);
+  }
+  for (size_t i = 0; i < info.verifyResult.sctsWithInvalidSignatures; ++i) {
+    glean::ssl::scts_verification_status.AccumulateSingleSample(3);
+  }
+  for (size_t i = 0; i < info.verifyResult.sctsWithInvalidTimestamps; ++i) {
+    glean::ssl::scts_verification_status.AccumulateSingleSample(4);
+  }
+
+  // See scts_origin in metrics.yaml.
+  for (size_t i = 0; i < info.verifyResult.embeddedSCTs; ++i) {
+    glean::ssl::scts_origin.AccumulateSingleSample(1);
+  }
+  for (size_t i = 0; i < info.verifyResult.sctsFromTLSHandshake; ++i) {
+    glean::ssl::scts_origin.AccumulateSingleSample(2);
+  }
+  for (size_t i = 0; i < info.verifyResult.sctsFromOCSP; ++i) {
+    glean::ssl::scts_origin.AccumulateSingleSample(3);
   }
 
   // Handle the histogram of SCTs counts.
@@ -501,43 +498,16 @@ void GatherCertificateTransparencyTelemetry(
       static_cast<uint32_t>(info.verifyResult.verifiedScts.size());
   // Note that sctsCount can also be 0 in case we've received SCT binary data,
   // but it failed to parse (e.g. due to unsupported CT protocol version).
-  Telemetry::Accumulate(Telemetry::SSL_SCTS_PER_CONNECTION, sctsCount);
-
-  // Report CT Policy compliance of EV certificates.
-  if (isEV) {
-    uint32_t evCompliance = 0;
-    switch (info.policyCompliance) {
-      case ct::CTPolicyCompliance::Compliant:
-        evCompliance = 1;
-        break;
-      case ct::CTPolicyCompliance::NotEnoughScts:
-        evCompliance = 2;
-        break;
-      case ct::CTPolicyCompliance::NotDiverseScts:
-        evCompliance = 3;
-        break;
-      case ct::CTPolicyCompliance::Unknown:
-      default:
-        MOZ_ASSERT_UNREACHABLE("Unexpected CTPolicyCompliance type");
-    }
-    Telemetry::Accumulate(Telemetry::SSL_CT_POLICY_COMPLIANCE_OF_EV_CERTS,
-                          evCompliance);
-  }
+  glean::ssl::scts_per_connection.AccumulateSingleSample(sctsCount);
 
   // Report CT Policy compliance by CA.
-  switch (info.policyCompliance) {
-    case ct::CTPolicyCompliance::Compliant:
-      AccumulateTelemetryForRootCA(
-          Telemetry::SSL_CT_POLICY_COMPLIANT_CONNECTIONS_BY_CA, rootCert);
-      break;
-    case ct::CTPolicyCompliance::NotEnoughScts:
-    case ct::CTPolicyCompliance::NotDiverseScts:
-      AccumulateTelemetryForRootCA(
-          Telemetry::SSL_CT_POLICY_NON_COMPLIANT_CONNECTIONS_BY_CA, rootCert);
-      break;
-    case ct::CTPolicyCompliance::Unknown:
-    default:
-      MOZ_ASSERT_UNREACHABLE("Unexpected CTPolicyCompliance type");
+  if (info.policyCompliance.isSome() &&
+      *info.policyCompliance != ct::CTPolicyCompliance::Compliant) {
+    int32_t binId = RootCABinNumber(rootCert);
+    if (binId != ROOT_CERTIFICATE_HASH_FAILURE) {
+      glean::ssl::ct_policy_non_compliant_connections_by_ca
+          .AccumulateSingleSample(binId);
+    }
   }
 }
 
@@ -550,38 +520,87 @@ static void CollectCertTelemetry(
     KeySizeStatus aKeySizeStatus,
     const PinningTelemetryInfo& aPinningTelemetryInfo,
     const nsTArray<nsTArray<uint8_t>>& aBuiltCertChain,
-    const CertificateTransparencyInfo& aCertificateTransparencyInfo) {
+    const CertificateTransparencyInfo& aCertificateTransparencyInfo,
+    const IssuerSources& issuerSources) {
   uint32_t evStatus = (aCertVerificationResult != Success) ? 0  // 0 = Failure
                       : (aEVStatus != EVStatus::EV)        ? 1  // 1 = DV
                                                            : 2;        // 2 = EV
-  Telemetry::Accumulate(Telemetry::CERT_EV_STATUS, evStatus);
+  glean::cert::ev_status.AccumulateSingleSample(evStatus);
 
   if (aOcspStaplingStatus != CertVerifier::OCSP_STAPLING_NEVER_CHECKED) {
-    Telemetry::Accumulate(Telemetry::SSL_OCSP_STAPLING, aOcspStaplingStatus);
+    glean::ssl::ocsp_stapling.AccumulateSingleSample(aOcspStaplingStatus);
   }
 
   if (aKeySizeStatus != KeySizeStatus::NeverChecked) {
-    Telemetry::Accumulate(Telemetry::CERT_CHAIN_KEY_SIZE_STATUS,
-                          static_cast<uint32_t>(aKeySizeStatus));
+    glean::cert::chain_key_size_status.AccumulateSingleSample(
+        static_cast<uint32_t>(aKeySizeStatus));
   }
 
   if (aPinningTelemetryInfo.accumulateForRoot) {
-    Telemetry::Accumulate(Telemetry::CERT_PINNING_FAILURES_BY_CA,
-                          aPinningTelemetryInfo.rootBucket);
+    glean::cert_pinning::failures_by_ca.AccumulateSingleSample(
+        aPinningTelemetryInfo.rootBucket);
   }
 
   if (aPinningTelemetryInfo.accumulateResult) {
-    MOZ_ASSERT(aPinningTelemetryInfo.certPinningResultHistogram.isSome());
-    Telemetry::Accumulate(
-        aPinningTelemetryInfo.certPinningResultHistogram.value(),
-        aPinningTelemetryInfo.certPinningResultBucket);
+    if (aPinningTelemetryInfo.isMoz) {
+      if (aPinningTelemetryInfo.testMode) {
+        glean::cert_pinning::moz_test_results_by_host.AccumulateSingleSample(
+            aPinningTelemetryInfo.certPinningResultBucket);
+      } else {
+        glean::cert_pinning::moz_results_by_host.AccumulateSingleSample(
+            aPinningTelemetryInfo.certPinningResultBucket);
+      }
+    } else {
+      if (aPinningTelemetryInfo.testMode) {
+        glean::cert_pinning::test_results
+            .EnumGet(static_cast<glean::cert_pinning::TestResultsLabel>(
+                aPinningTelemetryInfo.certPinningResultBucket))
+            .Add();
+      } else {
+        glean::cert_pinning::results
+            .EnumGet(static_cast<glean::cert_pinning::ResultsLabel>(
+                aPinningTelemetryInfo.certPinningResultBucket))
+            .Add();
+      }
+    }
   }
 
   if (aCertVerificationResult == Success && aBuiltCertChain.Length() > 0) {
     const nsTArray<uint8_t>& rootCert = aBuiltCertChain.LastElement();
-    AccumulateTelemetryForRootCA(Telemetry::CERT_VALIDATION_SUCCESS_BY_CA,
-                                 rootCert);
-    GatherCertificateTransparencyTelemetry(rootCert, aEVStatus == EVStatus::EV,
+    int32_t binId = RootCABinNumber(rootCert);
+    if (binId != ROOT_CERTIFICATE_HASH_FAILURE) {
+      glean::cert::validation_success_by_ca.AccumulateSingleSample(binId);
+    }
+
+    mozilla::glean::tls::certificate_verifications.Add(1);
+    if (issuerSources.contains(IssuerSource::TLSHandshake)) {
+      mozilla::glean::verification_used_cert_from::tls_handshake.AddToNumerator(
+          1);
+    }
+    if (issuerSources.contains(IssuerSource::PreloadedIntermediates)) {
+      mozilla::glean::verification_used_cert_from::preloaded_intermediates
+          .AddToNumerator(1);
+    }
+    if (issuerSources.contains(IssuerSource::ThirdPartyCertificates)) {
+      mozilla::glean::verification_used_cert_from::third_party_certificates
+          .AddToNumerator(1);
+    }
+    if (issuerSources.contains(IssuerSource::NSSCertDB)) {
+      mozilla::glean::verification_used_cert_from::nss_cert_db.AddToNumerator(
+          1);
+    }
+    if (issuerSources.contains(IssuerSource::BuiltInRootsModule)) {
+      mozilla::glean::verification_used_cert_from::built_in_roots_module
+          .AddToNumerator(1);
+    }
+  }
+
+  if ((aCertVerificationResult == Success ||
+       aCertVerificationResult ==
+           Result::ERROR_INSUFFICIENT_CERTIFICATE_TRANSPARENCY) &&
+      aBuiltCertChain.Length() > 0) {
+    const nsTArray<uint8_t>& rootCert = aBuiltCertChain.LastElement();
+    GatherCertificateTransparencyTelemetry(rootCert,
                                            aCertificateTransparencyInfo);
   }
 }
@@ -615,17 +634,18 @@ Result AuthCertificate(
         [](const auto& elementArray) { return elementArray.Clone(); });
   }
 
+  IssuerSources issuerSources;
   Result rv = certVerifier.VerifySSLServerCert(
       certBytes, time, aPinArg, aHostName, builtCertChain, certVerifierFlags,
       Some(std::move(peerCertsBytes)), stapledOCSPResponse,
       sctsFromTLSExtension, dcInfo, aOriginAttributes, &evStatus,
       &ocspStaplingStatus, &keySizeStatus, &pinningTelemetryInfo,
       &certificateTransparencyInfo, &aIsBuiltCertChainRootBuiltInRoot,
-      &aMadeOCSPRequests);
+      &aMadeOCSPRequests, &issuerSources);
 
   CollectCertTelemetry(rv, evStatus, ocspStaplingStatus, keySizeStatus,
                        pinningTelemetryInfo, builtCertChain,
-                       certificateTransparencyInfo);
+                       certificateTransparencyInfo, issuerSources);
 
   return rv;
 }
@@ -639,7 +659,7 @@ PRErrorCode AuthCertificateParseResults(
     nsITransportSecurityInfo::OverridableErrorCategory&
         aOverridableErrorCategory) {
   uint32_t probeValue = MapCertErrorToProbeValue(aCertVerificationError);
-  Telemetry::Accumulate(Telemetry::SSL_CERT_VERIFICATION_ERRORS, probeValue);
+  glean::ssl::cert_verification_errors.AccumulateSingleSample(probeValue);
 
   Maybe<nsITransportSecurityInfo::OverridableErrorCategory>
       maybeOverridableErrorCategory =
@@ -682,7 +702,7 @@ PRErrorCode AuthCertificateParseResults(
   if (haveOverride) {
     uint32_t probeValue =
         MapOverridableErrorToProbeValue(aCertVerificationError);
-    Telemetry::Accumulate(Telemetry::SSL_CERT_ERROR_OVERRIDES, probeValue);
+    glean::ssl::cert_error_overrides.AccumulateSingleSample(probeValue);
     MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
             ("[0x%" PRIx64 "] certificate error overridden", aPtrForLog));
     return 0;
@@ -692,12 +712,12 @@ PRErrorCode AuthCertificateParseResults(
 }
 
 static nsTArray<nsTArray<uint8_t>> CreateCertBytesArray(
-    const UniqueCERTCertList& aCertChain) {
+    const UniqueSECItemArray& aCertChain) {
   nsTArray<nsTArray<uint8_t>> certsBytes;
-  for (CERTCertListNode* n = CERT_LIST_HEAD(aCertChain);
-       !CERT_LIST_END(n, aCertChain); n = CERT_LIST_NEXT(n)) {
+  for (size_t i = 0; i < aCertChain->len; i++) {
     nsTArray<uint8_t> certBytes;
-    certBytes.AppendElements(n->cert->derCert.data, n->cert->derCert.len);
+    certBytes.AppendElements(aCertChain->items[i].data,
+                             aCertChain->items[i].len);
     certsBytes.AppendElement(std::move(certBytes));
   }
   return certsBytes;
@@ -754,14 +774,15 @@ SSLServerCertVerificationJob::Run() {
   // Runs on a cert verification thread and only on parent process.
   MOZ_ASSERT(XRE_IsParentProcess());
 
-  MOZ_LOG(
-      gPIPNSSLog, LogLevel::Debug,
-      ("[%" PRIx64 "] SSLServerCertVerificationJob::Run\n", mAddrForLogging));
+  MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
+          ("[%" PRIx64 "] SSLServerCertVerificationJob::Run", mAddrForLogging));
 
   RefPtr<SharedCertVerifier> certVerifier(GetDefaultCertVerifier());
   if (!certVerifier) {
-    PR_SetError(SEC_ERROR_NOT_INITIALIZED, 0);
-    return NS_OK;
+    // We can't release this off the STS thread because some parts of it
+    // are not threadsafe. Just leak mResultTask.
+    Unused << mResultTask.forget();
+    return NS_ERROR_FAILURE;
   }
 
   TimeStamp jobStartTime = TimeStamp::Now();
@@ -771,34 +792,38 @@ SSLServerCertVerificationJob::Run() {
   bool madeOCSPRequests = false;
   nsTArray<nsTArray<uint8_t>> builtChainBytesArray;
   nsTArray<uint8_t> certBytes(mPeerCertChain.ElementAt(0).Clone());
-  Result rv = AuthCertificate(
+  Result result = AuthCertificate(
       *certVerifier, mPinArg, certBytes, mPeerCertChain, mHostName,
       mOriginAttributes, mStapledOCSPResponse, mSCTsFromTLSExtension, mDCInfo,
       mProviderFlags, mTime, mCertVerifierFlags, builtChainBytesArray, evStatus,
       certificateTransparencyInfo, isCertChainRootBuiltInRoot,
       madeOCSPRequests);
 
-  if (rv == Success) {
-    Telemetry::AccumulateTimeDelta(
-        Telemetry::SSL_SUCCESFUL_CERT_VALIDATION_TIME_MOZILLAPKIX, jobStartTime,
-        TimeStamp::Now());
-    Telemetry::Accumulate(Telemetry::SSL_CERT_ERROR_OVERRIDES, 1);
+  TimeDuration elapsed = TimeStamp::Now() - jobStartTime;
+  if (result == Success) {
+    mozilla::glean::cert_verification_time::success.AccumulateRawDuration(
+        elapsed);
+    glean::ssl::cert_error_overrides.AccumulateSingleSample(1);
 
-    mResultTask->Dispatch(
+    nsresult rv = mResultTask->Dispatch(
         std::move(builtChainBytesArray), std::move(mPeerCertChain),
         TransportSecurityInfo::ConvertCertificateTransparencyInfoToStatus(
             certificateTransparencyInfo),
         evStatus, true, 0,
         nsITransportSecurityInfo::OverridableErrorCategory::ERROR_UNSET,
         isCertChainRootBuiltInRoot, mProviderFlags, madeOCSPRequests);
-    return NS_OK;
+    if (NS_FAILED(rv)) {
+      // We can't release this off the STS thread because some parts of it
+      // are not threadsafe. Just leak mResultTask.
+      Unused << mResultTask.forget();
+    }
+    return rv;
   }
 
-  Telemetry::AccumulateTimeDelta(
-      Telemetry::SSL_INITIAL_FAILED_CERT_VALIDATION_TIME_MOZILLAPKIX,
-      jobStartTime, TimeStamp::Now());
+  mozilla::glean::cert_verification_time::failure.AccumulateRawDuration(
+      elapsed);
 
-  PRErrorCode error = MapResultToPRErrorCode(rv);
+  PRErrorCode error = MapResultToPRErrorCode(result);
   nsITransportSecurityInfo::OverridableErrorCategory overridableErrorCategory =
       nsITransportSecurityInfo::OverridableErrorCategory::ERROR_UNSET;
   nsCOMPtr<nsIX509Cert> cert(new nsNSSCertificate(std::move(certBytes)));
@@ -807,12 +832,23 @@ SSLServerCertVerificationJob::Run() {
       overridableErrorCategory);
 
   // NB: finalError may be 0 here, in which the connection will continue.
-  mResultTask->Dispatch(
+  nsresult rv = mResultTask->Dispatch(
       std::move(builtChainBytesArray), std::move(mPeerCertChain),
-      nsITransportSecurityInfo::CERTIFICATE_TRANSPARENCY_NOT_APPLICABLE,
-      EVStatus::NotEV, false, finalError, overridableErrorCategory, false,
+      TransportSecurityInfo::ConvertCertificateTransparencyInfoToStatus(
+          certificateTransparencyInfo),
+      EVStatus::NotEV, false, finalError, overridableErrorCategory,
+      // If the certificate verifier returned Result::ERROR_BAD_CERT_DOMAIN,
+      // a chain was built, so isCertChainRootBuiltInRoot is valid and
+      // potentially useful. Otherwise, assume no chain was built.
+      result == Result::ERROR_BAD_CERT_DOMAIN ? isCertChainRootBuiltInRoot
+                                              : false,
       mProviderFlags, madeOCSPRequests);
-  return NS_OK;
+  if (NS_FAILED(rv)) {
+    // We can't release this off the STS thread because some parts of it
+    // are not threadsafe. Just leak mResultTask.
+    Unused << mResultTask.forget();
+  }
+  return rv;
 }
 
 // Takes information needed for cert verification, does some consistency
@@ -909,11 +945,15 @@ SECStatus AuthCertificateHook(void* arg, PRFileDesc* fd, PRBool checkSig,
     return SECFailure;
   }
 
-  UniqueCERTCertList peerCertChain(SSL_PeerCertificateChain(fd));
-  if (!peerCertChain) {
+  UniqueSECItemArray peerCertChain;
+  SECStatus rv =
+      SSL_PeerCertificateChainDER(fd, TempPtrToSetter(&peerCertChain));
+  if (rv != SECSuccess) {
     PR_SetError(PR_INVALID_STATE_ERROR, 0);
     return SECFailure;
   }
+  MOZ_ASSERT(peerCertChain,
+             "AuthCertificateHook: peerCertChain unexpectedly null");
 
   nsTArray<nsTArray<uint8_t>> peerCertsBytes =
       CreateCertBytesArray(peerCertChain);
@@ -944,16 +984,16 @@ SECStatus AuthCertificateHook(void* arg, PRFileDesc* fd, PRBool checkSig,
   socketInfo->GetProviderFlags(&providerFlags);
 
   uint32_t certVerifierFlags = 0;
-  if (!socketInfo->SharedState().IsOCSPStaplingEnabled() ||
-      !socketInfo->SharedState().IsOCSPMustStapleEnabled()) {
+  if (!StaticPrefs::security_ssl_enable_ocsp_stapling() ||
+      !StaticPrefs::security_ssl_enable_ocsp_must_staple()) {
     certVerifierFlags |= CertVerifier::FLAG_TLS_IGNORE_STATUS_REQUEST;
   }
 
   // Get DC information
   Maybe<DelegatedCredentialInfo> dcInfo;
   SSLPreliminaryChannelInfo channelPreInfo;
-  SECStatus rv = SSL_GetPreliminaryChannelInfo(fd, &channelPreInfo,
-                                               sizeof(channelPreInfo));
+  rv = SSL_GetPreliminaryChannelInfo(fd, &channelPreInfo,
+                                     sizeof(channelPreInfo));
   if (rv != SECSuccess) {
     PR_SetError(PR_INVALID_STATE_ERROR, 0);
     return SECFailure;
@@ -1003,11 +1043,8 @@ SECStatus AuthCertificateHookWithInfo(
   }
 
   uint32_t certVerifierFlags = 0;
-  // QuicSocketControl does not have a SharedState as NSSSocketControl.
-  // Here we need prefs for ocsp. This are prefs they are the same for
-  // PublicSSLState and PrivateSSLState, just take them from one of them.
-  if (!PublicSSLState()->IsOCSPStaplingEnabled() ||
-      !PublicSSLState()->IsOCSPMustStapleEnabled()) {
+  if (!StaticPrefs::security_ssl_enable_ocsp_stapling() ||
+      !StaticPrefs::security_ssl_enable_ocsp_must_staple()) {
     certVerifierFlags |= CertVerifier::FLAG_TLS_IGNORE_STATUS_REQUEST;
   }
 
@@ -1035,7 +1072,7 @@ SSLServerCertVerificationResult::SSLServerCertVerificationResult(
           nsITransportSecurityInfo::OverridableErrorCategory::ERROR_UNSET),
       mProviderFlags(0) {}
 
-void SSLServerCertVerificationResult::Dispatch(
+nsresult SSLServerCertVerificationResult::Dispatch(
     nsTArray<nsTArray<uint8_t>>&& aBuiltChain,
     nsTArray<nsTArray<uint8_t>>&& aPeerCertChain,
     uint16_t aCertificateTransparencyStatus, EVStatus aEVStatus,
@@ -1055,15 +1092,22 @@ void SSLServerCertVerificationResult::Dispatch(
   mProviderFlags = aProviderFlags;
   mMadeOCSPRequests = aMadeOCSPRequests;
 
-  if (mSucceeded && mBuiltChain.IsEmpty()) {
+  if (mSucceeded &&
+      (mBuiltChain.IsEmpty() || mFinalError != 0 ||
+       mOverridableErrorCategory !=
+           nsITransportSecurityInfo::OverridableErrorCategory::ERROR_UNSET)) {
     MOZ_ASSERT_UNREACHABLE(
-        "if the handshake succeeded, the built chain shouldn't be empty");
+        "if certificate verification succeeded without overridden errors, the "
+        "built chain shouldn't be empty and any error bits should be unset");
     mSucceeded = false;
     mFinalError = SEC_ERROR_LIBRARY_FAILURE;
   }
+  // Note that mSucceeded can be false while mFinalError is 0, in which case
+  // the connection will proceed.
   if (!mSucceeded && mPeerCertChain.IsEmpty()) {
     MOZ_ASSERT_UNREACHABLE(
-        "if the handshake failed, the peer chain shouldn't be empty");
+        "if certificate verification failed, the peer chain shouldn't be "
+        "empty");
     mFinalError = SEC_ERROR_LIBRARY_FAILURE;
   }
 
@@ -1071,9 +1115,15 @@ void SSLServerCertVerificationResult::Dispatch(
   nsCOMPtr<nsIEventTarget> stsTarget =
       do_GetService(NS_SOCKETTRANSPORTSERVICE_CONTRACTID, &rv);
   MOZ_ASSERT(stsTarget, "Failed to get socket transport service event target");
+  if (!stsTarget) {
+    // This has to be released on STS; just leak it
+    Unused << mSocketControl.forget();
+    return NS_ERROR_FAILURE;
+  }
   rv = stsTarget->Dispatch(this, NS_DISPATCH_NORMAL);
   MOZ_ASSERT(NS_SUCCEEDED(rv),
              "Failed to dispatch SSLServerCertVerificationResult");
+  return rv;
 }
 
 NS_IMETHODIMP
@@ -1090,13 +1140,11 @@ SSLServerCertVerificationResult::Run() {
   MOZ_ASSERT(onSTSThread);
 #endif
 
-  if (mSucceeded && !XRE_IsSocketProcess() &&
-      !(mProviderFlags & nsISocketProvider::NO_PERMANENT_STORAGE)) {
-    // This dispatches an event that will run when the socket thread is idle.
-    SaveIntermediateCerts(mBuiltChain);
-  }
-
   mSocketControl->SetMadeOCSPRequests(mMadeOCSPRequests);
+  mSocketControl->SetIsBuiltCertChainRootBuiltInRoot(
+      mIsBuiltCertChainRootBuiltInRoot);
+  mSocketControl->SetCertificateTransparencyStatus(
+      mCertificateTransparencyStatus);
 
   if (mSucceeded) {
     MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
@@ -1105,24 +1153,21 @@ SSLServerCertVerificationResult::Run() {
     nsCOMPtr<nsIX509Cert> cert(new nsNSSCertificate(std::move(certBytes)));
     mSocketControl->SetServerCert(cert, mEVStatus);
     mSocketControl->SetSucceededCertChain(std::move(mBuiltChain));
-
-    mSocketControl->SetIsBuiltCertChainRootBuiltInRoot(
-        mIsBuiltCertChainRootBuiltInRoot);
-    mSocketControl->SetCertificateTransparencyStatus(
-        mCertificateTransparencyStatus);
   } else {
     nsTArray<uint8_t> certBytes(mPeerCertChain.ElementAt(0).Clone());
     nsCOMPtr<nsIX509Cert> cert(new nsNSSCertificate(std::move(certBytes)));
-    // Certificate validation failed; store the peer certificate chain on
-    // mSocketControl so it can be used for error reporting.
+    mSocketControl->SetServerCert(cert, EVStatus::NotEV);
     mSocketControl->SetFailedCertChain(std::move(mPeerCertChain));
     if (mOverridableErrorCategory !=
         nsITransportSecurityInfo::OverridableErrorCategory::ERROR_UNSET) {
-      mSocketControl->SetStatusErrorBits(cert, mOverridableErrorCategory);
+      mSocketControl->SetStatusErrorBits(mOverridableErrorCategory);
     }
   }
 
   mSocketControl->SetCertVerificationResult(mFinalError);
+  // Release this reference to the socket control so that it will be freed on
+  // the socket thread.
+  mSocketControl = nullptr;
   return NS_OK;
 }
 
