@@ -18,7 +18,7 @@ Services.scriptloader.loadSubScript(
 add_task(async function () {
   // Ensure that XDG_CONFIG_HOME is there
   const xdgConfigHome = Services.env.get("XDG_CONFIG_HOME");
-  ok(xdgConfigHome.length > 1, "XDG_CONFIG_HOME is defined");
+  Assert.greater(xdgConfigHome.length, 1, "XDG_CONFIG_HOME is defined");
 
   // If it is there, do actual testing
   sanityChecks();
