@@ -9,18 +9,29 @@ import os
 import re
 from datetime import datetime, timedelta
 
+import requests
 from redo import retry
-from taskgraph.parameters import Parameters
-from taskgraph.target_tasks import get_method, register_target_task
-from taskgraph.util.taskcluster import find_task_id
+from taskgraph import create
+from taskgraph.target_tasks import register_target_task
+from taskgraph.util.attributes import attrmatch
+from taskgraph.util.parameterization import resolve_timestamps
+from taskgraph.util.taskcluster import (
+    find_task_id,
+    get_artifact,
+    get_task_definition,
+    parse_time,
+)
+from taskgraph.util.yaml import load_yaml
 
 from gecko_taskgraph import GECKO, try_option_syntax
 from gecko_taskgraph.util.attributes import (
+    is_try,
     match_run_on_hg_branches,
     match_run_on_projects,
 )
 from gecko_taskgraph.util.hg import find_hg_revision_push_info, get_hg_commit_message
 from gecko_taskgraph.util.platforms import platform_family
+from gecko_taskgraph.util.taskcluster import find_task, insert_index
 
 logger = logging.getLogger(__name__)
 
@@ -37,13 +48,13 @@ UNCOMMON_TRY_TASK_LABELS = [
     r"android-geckoview-docs",
     r"android-hw",
     # Windows tasks
-    r"windows11-64-2009-hw-ref",
+    r"windows11-64-24h2-hw-ref",
     r"windows10-aarch64-qr",
     # Linux tasks
     r"linux-",  # hide all linux32 tasks by default - bug 1599197
     r"linux1804-32",  # hide linux32 tests - bug 1599197
     # Test tasks
-    r"web-platform-tests(?!-webgpu).*backlog",  # hide wpt jobs that are not implemented yet - bug 1572820
+    r"web-platform-tests.*backlog",  # hide wpt jobs that are not implemented yet - bug 1572820
     r"-ccov",
     r"-profiling-",  # talos/raptor profiling jobs are run too often
     r"-32-.*-webgpu",  # webgpu gets little benefit from these tests.
@@ -148,6 +159,7 @@ def filter_release_tasks(task, parameters):
     if platform in (
         "linux",
         "linux64",
+        "linux64-aarch64",
         "macosx64",
         "win32",
         "win64",
@@ -255,9 +267,9 @@ def accept_raptor_android_build(platform):
         return False
     if "p6" in platform and "aarch64" in platform:
         return True
-    if "s21" in platform and "aarch64" in platform:
+    if "s24" in platform and "aarch64" in platform:
         return True
-    if "a51" in platform:
+    if "a55" in platform and "aarch64" in platform:
         return True
     return False
 
@@ -321,6 +333,9 @@ def _try_task_config(full_task_graph, parameters, graph_config):
         if "MOZHARNESS_TEST_PATHS" in parameters["try_task_config"].get("env", {}):
             matched_tasks = [x for x in matched_tasks if x.endswith("-1")]
 
+        if "MOZHARNESS_TEST_TAG" in parameters["try_task_config"].get("env", {}):
+            matched_tasks = [x for x in matched_tasks if x.endswith("-1")]
+
     selected_tasks = set(tasks) | set(matched_tasks)
     missing.update(selected_tasks - set(full_task_graph.tasks))
 
@@ -408,69 +423,6 @@ def target_tasks_try(full_task_graph, parameters, graph_config):
     return []
 
 
-@register_target_task("try_select_tasks")
-def target_tasks_try_select(full_task_graph, parameters, graph_config):
-    tasks = target_tasks_try_select_uncommon(full_task_graph, parameters, graph_config)
-    return [l for l in tasks if filter_by_uncommon_try_tasks(l)]
-
-
-@register_target_task("try_select_tasks_uncommon")
-def target_tasks_try_select_uncommon(full_task_graph, parameters, graph_config):
-    from gecko_taskgraph.decision import PER_PROJECT_PARAMETERS
-
-    projects = ("autoland", "mozilla-central")
-    if parameters["project"] not in projects:
-        projects = (parameters["project"],)
-
-    tasks = set()
-    for project in projects:
-        params = dict(parameters)
-        params["project"] = project
-        parameters = Parameters(**params)
-
-        try:
-            target_tasks_method = PER_PROJECT_PARAMETERS[project]["target_tasks_method"]
-        except KeyError:
-            target_tasks_method = "default"
-
-        tasks.update(
-            get_method(target_tasks_method)(full_task_graph, parameters, graph_config)
-        )
-
-    return sorted(tasks)
-
-
-@register_target_task("try_auto")
-def target_tasks_try_auto(full_task_graph, parameters, graph_config):
-    """Target the tasks which have indicated they should be run on autoland
-    (rather than try) via the `run_on_projects` attributes.
-
-    Should do the same thing as the `default` target tasks method.
-    """
-    params = dict(parameters)
-    params["project"] = "autoland"
-    parameters = Parameters(**params)
-
-    regex_filters = parameters["try_task_config"].get("tasks-regex")
-    include_regexes = exclude_regexes = []
-    if regex_filters:
-        include_regexes = [re.compile(r) for r in regex_filters.get("include", [])]
-        exclude_regexes = [re.compile(r) for r in regex_filters.get("exclude", [])]
-
-    return [
-        l
-        for l, t in full_task_graph.tasks.items()
-        if standard_filter(t, parameters)
-        and filter_out_shipping_phase(t, parameters)
-        and filter_out_devedition(t, parameters)
-        and filter_by_uncommon_try_tasks(t.label)
-        and filter_by_regex(t.label, include_regexes, mode="include")
-        and filter_by_regex(t.label, exclude_regexes, mode="exclude")
-        and filter_unsupported_artifact_builds(t, parameters)
-        and filter_out_shippable(t)
-    ]
-
-
 @register_target_task("default")
 def target_tasks_default(full_task_graph, parameters, graph_config):
     """Target the tasks which have indicated they should be run on this project
@@ -594,8 +546,8 @@ def target_tasks_mozilla_release(full_task_graph, parameters, graph_config):
     ]
 
 
-@register_target_task("mozilla_esr128_tasks")
-def target_tasks_mozilla_esr128(full_task_graph, parameters, graph_config):
+@register_target_task("mozilla_esr140_tasks")
+def target_tasks_mozilla_esr140(full_task_graph, parameters, graph_config):
     """Select the set of tasks required for a promotable beta or release build
     of desktop, without android CI. The candidates build process involves a pipeline
     of builds and signing, but does not include beetmover or balrog jobs."""
@@ -789,14 +741,46 @@ def target_tasks_custom_car_perf_testing(full_task_graph, parameters, graph_conf
 
         try_name = attributes.get("raptor_try_name")
 
+        if "network-bench" in try_name:
+            return False
+
         # Desktop and Android selection for CaR
-        if accept_raptor_desktop_build(platform) or accept_raptor_android_build(
-            platform
-        ):
-            if "browsertime" in try_name and (
-                "custom-car" in try_name or "cstm-car-m" in try_name
-            ):
-                if "hw-s21" in platform and "speedometer3" not in try_name:
+        if accept_raptor_desktop_build(platform):
+            if "browsertime" in try_name and "custom-car" in try_name:
+                # Bug 1898514: avoid tp6m or non-essential tp6 jobs in cron
+                if "tp6" in try_name and "essential" not in try_name:
+                    return False
+                # Bug 1928416
+                # For ARM coverage, this will only run on M2 machines at the moment.
+                if "jetstream2" in try_name:
+                    # Bug 1963732 - Disable js2 on 1500 mac for custom-car due to near perma
+                    if "m-car" in try_name and "1500" in platform:
+                        return False
+                    return True
+                return True
+        elif accept_raptor_android_build(platform):
+            if "browsertime" in try_name and "cstm-car-m" in try_name:
+                if "-nofis" not in try_name:
+                    return False
+                if "hw-s24" in platform and "speedometer3" not in try_name:
+                    return False
+                if "jetstream2" in try_name:
+                    return True
+                # Bug 1954124 - Don't run JS3 + Android on a cron yet.
+                if "jetstream3" in try_name:
+                    return False
+                # Bug 1898514 - Avoid tp6m or non-essential tp6 jobs in cron on non-a55 platform
+                # Bug 1961831 - Disable pageload tests temporarily during provider switch
+                if "tp6m" in try_name:
+                    return False
+                # Bug 1945165 - Disable ebay-kleinanzeigen on cstm-car-m because of permafail
+                if (
+                    "ebay-kleinanzeigen" in try_name
+                    and "ebay-kleinanzeigen-search" not in try_name
+                ):
+                    return False
+                # Bug 1960921 Disable ebay-kleinanzeigen-search-nofis on custom-car
+                if "ebay-kleinanzeigen-search-nofis" in try_name:
                     return False
                 return True
         return False
@@ -828,6 +812,9 @@ def target_tasks_general_perf_testing(full_task_graph, parameters, graph_config)
         if "live" in try_name and "sheriffed" not in try_name:
             return False
 
+        if "network-bench" in try_name:
+            return False
+
         # Desktop selection
         if accept_raptor_desktop_build(platform):
             # Select some browsertime tasks as desktop smoke-tests
@@ -835,6 +822,9 @@ def target_tasks_general_perf_testing(full_task_graph, parameters, graph_config)
                 # Disabled chrome responsiveness tests temporarily in bug 1898351
                 # due to frequent failures
                 return False
+            # Bug 1961145 - Disable bing-search for test-windows11-64-24h2-shippable
+            if "windows11" in platform and "bing-search" in try_name:
+                return False
             if "browsertime" in try_name:
                 if "chrome" in try_name:
                     if "tp6" in try_name and "essential" not in try_name:
@@ -851,12 +841,27 @@ def target_tasks_general_perf_testing(full_task_graph, parameters, graph_config)
                     if "speedometer" in try_name:
                         return True
                 if "safari" and "benchmark" in try_name:
+                    # Bug 1954202 Safari + JS3 seems to be perma failing on CI.
+                    if "jetstream3" in try_name and "safari-tp" not in try_name:
+                        return False
+                    if "jetstream2" in try_name and "safari" in try_name:
+                        return False
                     return True
         # Android selection
         elif accept_raptor_android_build(platform):
-            if "hw-s21" in platform and "speedometer3" not in try_name:
+            if "hw-s24" in platform and "speedometer3" not in try_name:
                 return False
-            if "chrome-m" in try_name and "essential" in try_name:
+            if "chrome-m" in try_name and "-nofis" not in try_name:
+                return False
+            # Bug 1929960 - Enable all chrome-m tp6m tests on a55 only
+            if "chrome-m" in try_name and "tp6m" in try_name and "hw-a55" in platform:
+                # Bug 1954923 - Disable ebay-kleinanzeigen
+                if (
+                    "ebay-kleinanzeigen" in try_name
+                    and "search" not in try_name
+                    and "nofis" in try_name
+                ):
+                    return False
                 return True
             if "chrome-m" in try_name and (
                 ("ebay" in try_name and "live" not in try_name)
@@ -869,39 +874,26 @@ def target_tasks_general_perf_testing(full_task_graph, parameters, graph_config)
             # Ignore all fennec tests here, we run those weekly
             if "fennec" in try_name:
                 return False
-            # Only run webrender tests
-            if "chrome-m" not in try_name and "-qr" not in platform:
-                return False
             # Select live site tests
             if "-live" in try_name:
                 return True
             # Select fenix resource usage tests
             if "fenix" in try_name:
-                # Bug 1816421 disable fission perf tests
-                if "-fis" in try_name:
-                    return False
                 if "-power" in try_name:
                     return True
-            # Select geckoview resource usage tests
+
             if "geckoview" in try_name:
-                # Bug 1816421 disable fission perf tests
-                if "-fis" in try_name:
-                    return False
-                # Run cpu+memory, and power tests
-                cpu_n_memory_task = "-cpu" in try_name and "-memory" in try_name
-                power_task = "-power" in try_name
-                # Ignore cpu+memory+power tests
-                if power_task and cpu_n_memory_task:
-                    return False
-                if cpu_n_memory_task:
-                    return False
-                if power_task:
-                    return "browsertime" in try_name
+                return False
             # Select browsertime-specific tests
             if "browsertime" in try_name:
                 # Don't run android CaR sp tests as we already have a cron for this.
                 if "m-car" in try_name:
                     return False
+                if "jetstream2" in try_name:
+                    return True
+                # Bug 1954124 - Don't run JS3 + Android on a cron yet.
+                if "jetstream3" in try_name:
+                    return False
                 if "fenix" in try_name:
                     return False
                 if "speedometer" in try_name:
@@ -914,6 +906,32 @@ def target_tasks_general_perf_testing(full_task_graph, parameters, graph_config)
     return [l for l, t in full_task_graph.tasks.items() if filter(t)]
 
 
+@register_target_task("geckoview-perftest")
+def target_tasks_geckoview_perftest(full_task_graph, parameters, graph_config):
+    """
+    Select tasks required for running geckoview tests 2 times a week.
+    """
+
+    def filter(task):
+        platform = task.attributes.get("test_platform")
+        attributes = task.attributes
+        if attributes.get("unittest_suite") != "raptor":
+            return False
+
+        if accept_raptor_android_build(platform):
+            try_name = attributes.get("raptor_try_name")
+            if "geckoview" in try_name and "browsertime" in try_name:
+                if "hw-s24" in platform and "speedometer" not in try_name:
+                    return False
+                if "live" in try_name and "cnn-amp" not in try_name:
+                    return False
+                return True
+
+        return False
+
+    return [l for l, t in full_task_graph.tasks.items() if filter(t)]
+
+
 def make_desktop_nightly_filter(platforms):
     """Returns a filter that gets all nightly tasks on the given platform."""
 
@@ -941,18 +959,25 @@ def target_tasks_speedometer_tests(full_task_graph, parameters, graph_config):
         if attributes.get("unittest_suite") != "raptor":
             return False
 
-        if accept_raptor_desktop_build(platform) or accept_raptor_android_build(
-            platform
-        ):
-            try_name = attributes.get("raptor_try_name")
-            if "hw-s21" in platform and "speedometer3" not in try_name:
-                return False
+        try_name = attributes.get("raptor_try_name")
+        if accept_raptor_desktop_build(platform):
             if (
                 "browsertime" in try_name
                 and "speedometer" in try_name
                 and "chrome" in try_name
             ):
                 return True
+        if accept_raptor_android_build(platform):
+            if "hw-s24" in platform and "speedometer3" not in try_name:
+                return False
+            if (
+                "browsertime" in try_name
+                and "speedometer" in try_name
+                and "chrome-m" in try_name
+            ):
+                if "-nofis" not in try_name:
+                    return False
+                return True
 
     return [l for l, t in full_task_graph.tasks.items() if filter(t)]
 
@@ -1092,12 +1117,44 @@ def target_tasks_nightly_all(full_task_graph, parameters, graph_config):
 @register_target_task("searchfox_index")
 def target_tasks_searchfox(full_task_graph, parameters, graph_config):
     """Select tasks required for indexing Firefox for Searchfox web site each day"""
+    index_path = (
+        f"{graph_config['trust-domain']}.v2.{parameters['project']}.revision."
+        f"{parameters['head_rev']}.searchfox-index"
+    )
+    if os.environ.get("MOZ_AUTOMATION"):
+        print(
+            f"Looking for existing index {index_path} to avoid triggering redundant indexing off the same revision..."
+        )
+        try:
+            task = find_task(index_path)
+            print(f"Index {index_path} exists: taskId {task['taskId']}")
+        except requests.exceptions.HTTPError as e:
+            if e.response.status_code != 404:
+                raise
+            print(f"Index {index_path} doesn't exist.")
+        else:
+            # Find the earlier expiration time of existing tasks
+            taskdef = get_task_definition(task["taskId"])
+            task_graph = get_artifact(task["taskId"], "public/task-graph.json")
+            if task_graph:
+                base_time = parse_time(taskdef["created"])
+                first_expiry = min(
+                    resolve_timestamps(base_time, t["task"]["expires"])
+                    for t in task_graph.values()
+                )
+                expiry = parse_time(first_expiry)
+                if expiry > datetime.utcnow() + timedelta(days=7):
+                    print("Skipping index tasks")
+                    return []
+        if not create.testing:
+            insert_index(index_path, os.environ["TASK_ID"], use_proxy=True)
+
     return [
         "searchfox-linux64-searchfox/debug",
         "searchfox-macosx64-searchfox/debug",
         "searchfox-macosx64-aarch64-searchfox/debug",
         "searchfox-win64-searchfox/debug",
-        "searchfox-android-armv7-searchfox/debug",
+        "searchfox-android-aarch64-searchfox/debug",
         "searchfox-ios-searchfox/debug",
         "source-test-file-metadata-bugzilla-components",
         "source-test-file-metadata-test-info-all",
@@ -1114,7 +1171,10 @@ def target_tasks_build_linux64_clang_trunk_perf(
 
     # Only keep tasks generated from platform `linux1804-64-clang-trunk-qr/opt`
     def filter(task_label):
-        if "linux1804-64-clang-trunk-qr/opt" in task_label:
+        # Bug 1961141 - Disable unity webgl for linux1804-64-clang-trunk-qr
+        if "linux1804-64-clang-trunk-qr/opt" in task_label and "unity" in task_label:
+            return False
+        if "linux1804-64-clang-trunk-qr/opt" in task_label and "live" not in task_label:
             return True
         return False
 
@@ -1191,40 +1251,12 @@ def target_tasks_bouncer_check(full_task_graph, parameters, graph_config):
     return [l for l, t in full_task_graph.tasks.items() if filter(t)]
 
 
-@register_target_task("staging_release_builds")
-def target_tasks_staging_release(full_task_graph, parameters, graph_config):
-    """
-    Select all builds that are part of releases.
-    """
-
-    def filter(task):
-        if not task.attributes.get("shipping_product"):
-            return False
-        if parameters["release_type"].startswith(
-            "esr"
-        ) and "android" in task.attributes.get("build_platform", ""):
-            return False
-        if parameters["release_type"] != "beta" and "devedition" in task.attributes.get(
-            "build_platform", ""
-        ):
-            return False
-        if task.attributes.get("shipping_phase") == "build":
-            return True
-        return False
-
-    return [l for l, t in full_task_graph.tasks.items() if filter(t)]
-
-
-@register_target_task("release_simulation")
-def target_tasks_release_simulation(full_task_graph, parameters, graph_config):
-    """
-    Select builds that would run on push on a release branch.
-    """
+def _filter_by_release_project(parameters):
     project_by_release = {
         "nightly": "mozilla-central",
         "beta": "mozilla-beta",
         "release": "mozilla-release",
-        "esr128": "mozilla-esr128",
+        "esr140": "mozilla-esr140",
     }
     target_project = project_by_release.get(parameters["release_type"])
     if target_project is None:
@@ -1235,12 +1267,38 @@ def target_tasks_release_simulation(full_task_graph, parameters, graph_config):
         run_on_projects = set(task.attributes.get("run_on_projects", []))
         return match_run_on_projects(target_project, run_on_projects)
 
-    def filter_out_android_on_esr(task):
-        if parameters["release_type"].startswith(
-            "esr"
-        ) and "android" in task.attributes.get("build_platform", ""):
-            return False
-        return True
+    return filter_for_target_project
+
+
+def filter_out_android_on_esr(parameters, task):
+    return not parameters["release_type"].startswith(
+        "esr"
+    ) or "android" not in task.attributes.get("build_platform", "")
+
+
+@register_target_task("staging_release_builds")
+def target_tasks_staging_release(full_task_graph, parameters, graph_config):
+    """
+    Select all builds that are part of releases.
+    """
+    filter_for_target_project = _filter_by_release_project(parameters)
+
+    return [
+        l
+        for l, t in full_task_graph.tasks.items()
+        if t.attributes.get("shipping_product")
+        and filter_out_android_on_esr(parameters, t)
+        and filter_for_target_project(t)
+        and t.attributes.get("shipping_phase") == "build"
+    ]
+
+
+@register_target_task("release_simulation")
+def target_tasks_release_simulation(full_task_graph, parameters, graph_config):
+    """
+    Select tasks that would run on push on a release branch.
+    """
+    filter_for_target_project = _filter_by_release_project(parameters)
 
     return [
         l
@@ -1248,7 +1306,7 @@ def target_tasks_release_simulation(full_task_graph, parameters, graph_config):
         if filter_release_tasks(t, parameters)
         and filter_out_cron(t, parameters)
         and filter_for_target_project(t)
-        and filter_out_android_on_esr(t)
+        and filter_out_android_on_esr(parameters, t)
     ]
 
 
@@ -1356,22 +1414,8 @@ def target_tasks_daily_beta_perf(full_task_graph, parameters, graph_config):
                 if "benchmark" in try_name:
                     return True
         elif accept_raptor_android_build(platform):
-            # Select browsertime & geckoview specific tests
             if "browsertime" and "geckoview" in try_name:
-                if "power" in try_name:
-                    return False
-                if "cpu" in try_name:
-                    return False
-                if "profiling" in try_name:
-                    return False
-                if "-live" in try_name:
-                    return False
-                if "speedometer" in try_name:
-                    return True
-                if "webgl" in try_name:
-                    return True
-                if "tp6m" in try_name:
-                    return True
+                return False
 
         return False
 
@@ -1422,24 +1466,8 @@ def target_tasks_weekly_release_perf(full_task_graph, parameters, graph_config):
                 if "youtube-playback" in try_name:
                     return True
         elif accept_raptor_android_build(platform):
-            # Select browsertime & geckoview specific tests
             if "browsertime" and "geckoview" in try_name:
-                if "power" in try_name:
-                    return False
-                if "cpu" in try_name:
-                    return False
-                if "profiling" in try_name:
-                    return False
-                if "-live" in try_name:
-                    return False
-                if "speedometer" in try_name:
-                    return True
-                if "webgl" in try_name:
-                    return True
-                if "tp6m" in try_name:
-                    return True
-                if "youtube-playback" in try_name:
-                    return True
+                return False
 
         return False
 
@@ -1592,16 +1620,6 @@ def target_tasks_perftest_autoland(full_task_graph, parameters, graph_config):
             yield name
 
 
-@register_target_task("l10n-cross-channel")
-def target_tasks_l10n_cross_channel(full_task_graph, parameters, graph_config):
-    """Select the set of tasks required to run l10n cross-channel."""
-
-    def filter(task):
-        return task.kind in ["l10n-cross-channel"]
-
-    return [l for l, t in full_task_graph.tasks.items() if filter(t)]
-
-
 @register_target_task("eslint-build")
 def target_tasks_eslint_build(full_task_graph, parameters, graph_config):
     """Select the task to run additional ESLint rules which require a build."""
@@ -1623,14 +1641,17 @@ def target_tasks_holly(full_task_graph, parameters, graph_config):
     return [l for l, t in full_task_graph.tasks.items() if filter(t)]
 
 
-@register_target_task("snap_upstream_tests")
-def target_tasks_snap_upstream_tests(full_task_graph, parameters, graph_config):
+@register_target_task("snap_upstream_tasks")
+def target_tasks_snap_upstream_tasks(full_task_graph, parameters, graph_config):
     """
-    Select tasks for testing Snap package built as upstream. Omit -try because
-    it does not really make sense on a m-c cron
+    Select tasks for building/testing Snap package built as upstream. Omit -try
+    because it does not really make sense on a m-c cron
+
+    Use test tasks for linux64 builds and only builds for arm* until there is
+    support for running tests (bug 1855463)
     """
     for name, task in full_task_graph.tasks.items():
-        if "snap-upstream-test" in name and not "-try" in name:
+        if "snap-upstream" in name and not "-local" in name:
             yield name
 
 
@@ -1683,3 +1704,42 @@ def target_tasks_android_l10n_import(full_task_graph, parameters, graph_config):
 @register_target_task("android-l10n-sync")
 def target_tasks_android_l10n_sync(full_task_graph, parameters, graph_config):
     return [l for l, t in full_task_graph.tasks.items() if l == "android-l10n-sync"]
+
+
+@register_target_task("os-integration")
+def target_tasks_os_integration(full_task_graph, parameters, graph_config):
+    candidate_attrs = load_yaml(
+        os.path.join(GECKO, "taskcluster", "kinds", "test", "os-integration.yml")
+    )
+
+    labels = []
+    for label, task in full_task_graph.tasks.items():
+        if task.kind not in ("test", "source-test", "perftest"):
+            continue
+
+        # Match tasks against attribute sets defined in os-integration.yml.
+        if not any(attrmatch(task.attributes, **c) for c in candidate_attrs):
+            continue
+
+        if not is_try(parameters):
+            # Only run hardware tasks if scheduled from try. We do this because
+            # the `cron` task is designed to provide a base for testing worker
+            # images, which isn't something that impacts our hardware pools.
+            if (
+                task.attributes.get("build_platform") == "macosx64"
+                or "android-hw" in label
+            ):
+                continue
+
+            # Perform additional filtering for non-try repos. We don't want to
+            # limit what can be scheduled on try as os-integration tests are still
+            # useful for manual verification of things.
+            if not (
+                filter_for_project(task, parameters)
+                and filter_for_hg_branch(task, parameters)
+                and filter_tests_without_manifests(task, parameters)
+            ):
+                continue
+
+        labels.append(label)
+    return labels
