@@ -30,9 +30,6 @@ beetmover_push_to_release_description_schema = Schema(
         Required("shipping-phase"): task_description_schema["shipping-phase"],
         Required("shipping-product"): task_description_schema["shipping-product"],
         Optional("extra"): task_description_schema["extra"],
-        Optional("worker"): {
-            Optional("max-run-time"): int,
-        },
     }
 )
 
@@ -85,8 +82,6 @@ def make_beetmover_push_to_release_worker(config, jobs):
             "implementation": "beetmover-push-to-release",
             "product": job["product"],
         }
-        if job.get("worker", {}).get("max-run-time"):
-            worker["max-run-time"] = job["worker"]["max-run-time"]
         job["worker"] = worker
         del job["product"]
 
