@@ -15,6 +15,8 @@ from gecko_taskgraph.transforms.job import configure_taskdesc_for_run, run_job_u
 from gecko_taskgraph.transforms.job.common import get_expiration, support_vcs_checkout
 from gecko_taskgraph.transforms.test import normpath, test_description_schema
 from gecko_taskgraph.util.attributes import is_try
+from gecko_taskgraph.util.chunking import get_test_tags
+from gecko_taskgraph.util.perftest import is_external_browser
 
 VARIANTS = [
     "shippable",
@@ -63,9 +65,13 @@ def test_packages_url(taskdesc):
     )
     # for android shippable we need to add 'en-US' to the artifact url
     test = taskdesc["run"]["test"]
-    if "android" in test["test-platform"] and (
-        get_variant(test["test-platform"])
-        in ("shippable", "shippable-qr", "shippable-lite", "shippable-lite-qr")
+    if (
+        "android" in test["test-platform"]
+        and (
+            get_variant(test["test-platform"])
+            in ("shippable", "shippable-qr", "shippable-lite", "shippable-lite-qr")
+        )
+        and not is_external_browser(test.get("try-name", ""))
     ):
         head, tail = os.path.split(artifact_url)
         artifact_url = os.path.join(head, "en-US", tail)
@@ -104,7 +110,7 @@ def mozharness_test_on_docker(config, job, taskdesc):
     worker["max-run-time"] = test["max-run-time"]
     worker["retry-exit-status"] = test["retry-exit-status"]
     if "android-em-7.0-x86" in test["test-platform"]:
-        worker["privileged"] = True
+        worker["kvm"] = True
 
     artifacts = [
         # (artifact name prefix, in-image path)
@@ -141,7 +147,6 @@ def mozharness_test_on_docker(config, job, taskdesc):
             "MOZHARNESS_CONFIG": " ".join(mozharness["config"]),
             "MOZHARNESS_SCRIPT": mozharness["script"],
             "MOZILLA_BUILD_URL": {"task-reference": installer},
-            "NEED_PULSEAUDIO": "true",
             "NEED_WINDOW_MANAGER": "true",
             "ENABLE_E10S": str(bool(test.get("e10s"))).lower(),
             "WORKING_DIR": "/builds/worker",
@@ -150,22 +155,25 @@ def mozharness_test_on_docker(config, job, taskdesc):
 
     env["PYTHON"] = "python3"
 
-    # Legacy linux64 tests rely on compiz.
-    if test.get("docker-image", {}).get("in-tree") == "desktop1604-test":
-        env.update({"NEED_COMPIZ": "true"})
-
-    # Bug 1602701/1601828 - use compiz on ubuntu1804 due to GTK asynchiness
-    # when manipulating windows.
     if test.get("docker-image", {}).get("in-tree") == "ubuntu1804-test":
+        env["NEED_PULSEAUDIO"] = "true"
+
+        # Bug 1602701/1601828 - use compiz on ubuntu1804 due to GTK asynchiness
+        # when manipulating windows.
         if "wdspec" in job["run"]["test"]["suite"] or (
             "marionette" in job["run"]["test"]["suite"]
             and "headless" not in job["label"]
         ):
             env.update({"NEED_COMPIZ": "true"})
 
+    if test.get("docker-image", {}).get("in-tree") == "ubuntu2404-test":
+        env["NEED_PIPEWIRE"] = "true"
+
     # Set MOZ_ENABLE_WAYLAND env variables to enable Wayland backend.
     if "wayland" in job["label"]:
         env["MOZ_ENABLE_WAYLAND"] = "1"
+    else:
+        assert "MOZ_ENABLE_WAYLAND" not in env
 
     if mozharness.get("mochitest-flavor"):
         env["MOCHITEST_FLAVOR"] = mozharness["mochitest-flavor"]
@@ -185,9 +193,10 @@ def mozharness_test_on_docker(config, job, taskdesc):
             "reboot: {} not supported on generic-worker".format(test["reboot"])
         )
 
-    # Support vcs checkouts regardless of whether the task runs from
-    # source or not in case it is needed on an interactive loaner.
-    support_vcs_checkout(config, job, taskdesc)
+    if not test["checkout"]:
+        # Support vcs checkouts regardless of whether the task runs from
+        # source or not in case it is needed on an interactive loaner.
+        support_vcs_checkout(config, job, taskdesc)
 
     # If we have a source checkout, run mozharness from it instead of
     # downloading a zip file with the same content.
@@ -225,6 +234,11 @@ def mozharness_test_on_docker(config, job, taskdesc):
             {test["suite"]: test["test-manifests"]}, sort_keys=True
         )
 
+    test_tags = get_test_tags(config, env)
+    if test_tags:
+        env["MOZHARNESS_TEST_TAG"] = json.dumps(test_tags)
+        command.extend([f"--tag={x}" for x in test_tags])
+
     # TODO: remove the need for run['chunked']
     elif mozharness.get("chunked") or test["chunks"] > 1:
         command.append("--total-chunk={}".format(test["chunks"]))
@@ -237,11 +251,13 @@ def mozharness_test_on_docker(config, job, taskdesc):
         )
         command.append("--download-symbols=" + download_symbols)
 
+    use_caches = test.get("use-caches", ["checkout", "pip", "uv"])
     job["run"] = {
         "workdir": run["workdir"],
         "tooltool-downloads": mozharness["tooltool-downloads"],
         "checkout": test["checkout"],
         "command": command,
+        "use-caches": use_caches,
         "using": "run-task",
     }
     configure_taskdesc_for_run(config, job, taskdesc, worker["implementation"])
@@ -257,8 +273,12 @@ def mozharness_test_on_generic_worker(config, job, taskdesc):
 
     is_macosx = worker["os"] == "macosx"
     is_windows = worker["os"] == "windows"
-    is_linux = worker["os"] == "linux" or worker["os"] == "linux-bitbar"
+    is_linux = worker["os"] == "linux" or worker["os"] in [
+        "linux-bitbar",
+        "linux-lambda",
+    ]
     is_bitbar = worker["os"] == "linux-bitbar"
+    is_lambda = worker["os"] == "linux-lambda"
     assert is_macosx or is_windows or is_linux
 
     artifacts = [
@@ -281,7 +301,7 @@ def mozharness_test_on_generic_worker(config, job, taskdesc):
             }
         )
 
-    if is_bitbar:
+    if is_bitbar or is_lambda:
         artifacts = [
             {
                 "name": "public/test/",
@@ -348,7 +368,7 @@ def mozharness_test_on_generic_worker(config, job, taskdesc):
                 "SHELL": "/bin/bash",
             }
         )
-    elif is_bitbar:
+    elif is_bitbar or is_lambda:
         env.update(
             {
                 "LANG": "en_US.UTF-8",
@@ -358,7 +378,6 @@ def mozharness_test_on_generic_worker(config, job, taskdesc):
                     "artifact-reference": "<build/public/build/mozharness.zip>"
                 },
                 "MOZILLA_BUILD_URL": {"task-reference": installer},
-                "MOZ_NO_REMOTE": "1",
                 "NEED_XVFB": "false",
                 "XPCOM_DEBUG_BREAK": "warn",
                 "NO_FAIL_ON_TEST_ERRORS": "1",
@@ -392,7 +411,7 @@ def mozharness_test_on_generic_worker(config, job, taskdesc):
             "-u",
             "mozharness\\scripts\\" + normpath(mozharness["script"]),
         ]
-    elif is_bitbar:
+    elif is_bitbar or is_lambda:
         py_binary = "python3"
         mh_command = ["bash", f"./{bitbar_script}"]
     elif is_macosx:
@@ -435,6 +454,13 @@ def mozharness_test_on_generic_worker(config, job, taskdesc):
             {test["suite"]: test["test-manifests"]}, sort_keys=True
         )
 
+    test_tags = get_test_tags(config, env)
+    if test_tags:
+        # do not add --tag for perf tests
+        if test["suite"] not in ["talos", "raptor"]:
+            env["MOZHARNESS_TEST_TAG"] = json.dumps(test_tags)
+            mh_command.extend([f"--tag={x}" for x in test_tags])
+
     # TODO: remove the need for run['chunked']
     elif mozharness.get("chunked") or test["chunks"] > 1:
         mh_command.append("--total-chunk={}".format(test["chunks"]))
@@ -453,7 +479,7 @@ def mozharness_test_on_generic_worker(config, job, taskdesc):
             "format": "zip",
         }
     ]
-    if is_bitbar:
+    if is_bitbar or is_lambda:
         a_url = config.params.file_url(
             f"taskcluster/scripts/tester/{bitbar_script}",
         )
@@ -466,12 +492,14 @@ def mozharness_test_on_generic_worker(config, job, taskdesc):
             }
         ]
 
+    use_caches = test.get("use-caches", ["checkout", "pip", "uv"])
     job["run"] = {
         "tooltool-downloads": mozharness["tooltool-downloads"],
         "checkout": test["checkout"],
         "command": mh_command,
+        "use-caches": use_caches,
         "using": "run-task",
     }
-    if is_bitbar:
+    if is_bitbar or is_lambda:
         job["run"]["run-as-root"] = True
     configure_taskdesc_for_run(config, job, taskdesc, worker["implementation"])
