@@ -18,7 +18,9 @@ import time
 import attr
 from mozbuild.util import memoize
 from taskcluster.utils import fromNow
+from taskgraph import MAX_DEPENDENCIES
 from taskgraph.transforms.base import TransformSequence
+from taskgraph.util.copy import deepcopy
 from taskgraph.util.keyed_by import evaluate_keyed_by
 from taskgraph.util.schema import (
     Schema,
@@ -30,12 +32,12 @@ from taskgraph.util.schema import (
 from taskgraph.util.treeherder import split_symbol
 from voluptuous import All, Any, Extra, Match, NotIn, Optional, Required
 
-from gecko_taskgraph import GECKO, MAX_DEPENDENCIES
+from gecko_taskgraph import GECKO
 from gecko_taskgraph.optimize.schema import OptimizationSchema
 from gecko_taskgraph.transforms.job.common import get_expiration
 from gecko_taskgraph.util import docker as dockerutil
 from gecko_taskgraph.util.attributes import TRUNK_PROJECTS, is_try, release_level
-from gecko_taskgraph.util.copy_task import copy_task
+from gecko_taskgraph.util.chunking import TEST_VARIANTS
 from gecko_taskgraph.util.hash import hash_path
 from gecko_taskgraph.util.partners import get_partners_to_be_published
 from gecko_taskgraph.util.scriptworker import BALROG_ACTIONS, get_release_config
@@ -71,7 +73,7 @@ task_description_schema = Schema(
         # attributes for this task
         Optional("attributes"): {str: object},
         # relative path (from config.path) to the file task was defined in
-        Optional("job-from"): str,
+        Optional("task-from"): str,
         # dependencies of this task, keyed by name; these are passed through
         # verbatim and subject to the interpretation of the Task's get_dependencies
         # method.
@@ -146,8 +148,8 @@ task_description_schema = Schema(
             # 'by-tier' behavior will be used.
             "rank": Any(
                 # Rank is equal the timestamp of the build_date for tier-1
-                # tasks, and zero for non-tier-1.  This sorts tier-{2,3}
-                # builds below tier-1 in the index.
+                # tasks, and one for non-tier-1.  This sorts tier-{2,3}
+                # builds below tier-1 in the index, but above eager-index.
                 "by-tier",
                 # Rank is given as an integer constant (e.g. zero to make
                 # sure a task is last in the index).
@@ -210,7 +212,7 @@ TC_TREEHERDER_SCHEMA_URL = (
 
 
 UNKNOWN_GROUP_NAME = (
-    "Treeherder group {} (from {}) has no name; " "add it to taskcluster/ci/config.yml"
+    "Treeherder group {} (from {}) has no name; " "add it to taskcluster/config.yml"
 )
 
 V2_ROUTE_TEMPLATES = [
@@ -219,6 +221,7 @@ V2_ROUTE_TEMPLATES = [
     "index.{trust-domain}.v2.{project}.pushdate.{build_date}.latest.{product}.{job-name}",
     "index.{trust-domain}.v2.{project}.pushlog-id.{pushlog_id}.{product}.{job-name}",
     "index.{trust-domain}.v2.{project}.revision.{branch_rev}.{product}.{job-name}",
+    "index.{trust-domain}.v2.{project}.revision.{branch_git_rev}.{product}.{job-name}",
 ]
 
 # {central, inbound, autoland} write to a "trunk" index prefix. This facilitates
@@ -232,6 +235,7 @@ V2_SHIPPABLE_TEMPLATES = [
     "index.{trust-domain}.v2.{project}.shippable.{build_date}.revision.{branch_rev}.{product}.{job-name}",  # noqa - too long
     "index.{trust-domain}.v2.{project}.shippable.{build_date}.latest.{product}.{job-name}",
     "index.{trust-domain}.v2.{project}.shippable.revision.{branch_rev}.{product}.{job-name}",
+    "index.{trust-domain}.v2.{project}.shippable.revision.{branch_git_rev}.{product}.{job-name}",
 ]
 
 V2_SHIPPABLE_L10N_TEMPLATES = [
@@ -262,6 +266,12 @@ def get_branch_rev(config):
     ]
 
 
+def get_branch_git_rev(config):
+    return config.params[
+        "{}head_git_rev".format(config.graph_config["project-repo-param-prefix"])
+    ]
+
+
 def get_branch_repo(config):
     return config.params[
         "{}head_repository".format(
@@ -293,6 +303,7 @@ def payload_builder(name, schema):
     )
 
     def wrap(func):
+        assert name not in payload_builders, f"duplicate payload builder name {name}"
         payload_builders[name] = PayloadBuilder(schema, func)
         return func
 
@@ -305,6 +316,7 @@ index_builders = {}
 
 def index_builder(name):
     def wrap(func):
+        assert name not in index_builders, f"duplicate index builder name {name}"
         index_builders[name] = func
         return func
 
@@ -313,7 +325,7 @@ def index_builder(name):
 
 UNSUPPORTED_INDEX_PRODUCT_ERROR = """\
 The gecko-v2 product {product} is not in the list of configured products in
-`taskcluster/ci/config.yml'.
+`taskcluster/config.yml'.
 """
 
 
@@ -347,6 +359,7 @@ def verify_index(config, index):
         Required("loopback-audio"): bool,
         Required("docker-in-docker"): bool,  # (aka 'dind')
         Required("privileged"): bool,
+        Optional("kvm"): bool,
         # Paths to Docker volumes.
         #
         # For in-tree Docker images, volumes can be parsed from Dockerfile.
@@ -490,6 +503,11 @@ def build_docker_worker_payload(config, task, task_def):
             devices[capitalized] = True
             task_def["scopes"].append("docker-worker:capability:device:" + capitalized)
 
+    if worker.get("kvm"):
+        devices = capabilities.setdefault("devices", {})
+        devices["kvm"] = True
+        task_def["scopes"].append("docker-worker:capability:device:kvm")
+
     if worker.get("privileged"):
         capabilities["privileged"] = True
         task_def["scopes"].append("docker-worker:capability:privileged")
@@ -622,7 +640,9 @@ def build_docker_worker_payload(config, task, task_def):
 @payload_builder(
     "generic-worker",
     schema={
-        Required("os"): Any("windows", "macosx", "linux", "linux-bitbar"),
+        Required("os"): Any(
+            "windows", "macosx", "linux", "linux-bitbar", "linux-lambda"
+        ),
         # see http://schemas.taskcluster.net/generic-worker/v1/payload.json
         # and https://docs.taskcluster.net/reference/workers/generic-worker/payload
         # command is a list of commands to run, sequentially
@@ -677,7 +697,7 @@ def build_docker_worker_payload(config, task, task_def):
                 # Required if and only if `content` is specified and mounting a
                 # directory (not a file). This should be the archive format of the
                 # content (either pre-loaded cache or read-only directory).
-                Optional("format"): Any("rar", "tar.bz2", "tar.gz", "zip"),
+                Optional("format"): Any("rar", "tar.bz2", "tar.gz", "zip", "tar.xz"),
             }
         ],
         # environment variables
@@ -719,7 +739,7 @@ def build_generic_worker_payload(config, task, task_def):
         task_def["payload"].setdefault("onExitStatus", {}).setdefault(
             "retry", []
         ).extend(worker["retry-exit-status"])
-    if worker["os"] == "linux-bitbar":
+    if worker["os"] in ["linux-bitbar", "linux-lambda"]:
         task_def["payload"].setdefault("onExitStatus", {}).setdefault("retry", [])
         # exit code 4 is used to indicate an intermittent android device error
         if 4 not in task_def["payload"]["onExitStatus"]["retry"]:
@@ -766,7 +786,7 @@ def build_generic_worker_payload(config, task, task_def):
     #   * 'cache-name' -> 'cacheName'
     #   * 'task-id'    -> 'taskId'
     # All other key names are already suitable, and don't need renaming.
-    mounts = copy_task(worker.get("mounts", []))
+    mounts = deepcopy(worker.get("mounts", []))
     for mount in mounts:
         if "cache-name" in mount:
             mount["cacheName"] = "{trust_domain}-level-{level}-{name}".format(
@@ -818,7 +838,7 @@ def build_generic_worker_payload(config, task, task_def):
     "scriptworker-signing",
     schema={
         # the maximum time to run, in seconds
-        Required("max-run-time"): int,
+        Optional("max-run-time"): int,
         # list of artifact URLs for the artifacts that should be signed
         Required("upstream-artifacts"): [
             {
@@ -836,7 +856,9 @@ def build_generic_worker_payload(config, task, task_def):
         # behavior for mac iscript
         Optional("mac-behavior"): Any(
             "apple_notarization",
+            "apple_notarization_stacked",
             "mac_sign_and_pkg",
+            "mac_sign_and_pkg_hardened",
             "mac_geckodriver",
             "mac_notarize_geckodriver",
             "mac_single_file",
@@ -844,18 +866,38 @@ def build_generic_worker_payload(config, task, task_def):
         ),
         Optional("entitlements-url"): str,
         Optional("requirements-plist-url"): str,
+        Optional("provisioning-profile-config"): [
+            {
+                Required("profile_name"): str,
+                Required("target_path"): str,
+            }
+        ],
+        Optional("hardened-sign-config"): [
+            {
+                Optional("deep"): bool,
+                Optional("runtime"): bool,
+                Optional("force"): bool,
+                Optional("entitlements"): str,
+                Optional("requirements"): str,
+                Required("globs"): [str],
+            }
+        ],
     },
 )
 def build_scriptworker_signing_payload(config, task, task_def):
     worker = task["worker"]
 
     task_def["payload"] = {
-        "maxRunTime": worker["max-run-time"],
         "upstreamArtifacts": worker["upstream-artifacts"],
     }
     if worker.get("mac-behavior"):
         task_def["payload"]["behavior"] = worker["mac-behavior"]
-        for attribute in ("entitlements-url", "requirements-plist-url"):
+        for attribute in (
+            "entitlements-url",
+            "requirements-plist-url",
+            "hardened-sign-config",
+            "provisioning-profile-config",
+        ):
             if worker.get(attribute):
                 task_def["payload"][attribute] = worker[attribute]
 
@@ -876,10 +918,9 @@ def build_scriptworker_signing_payload(config, task, task_def):
     "beetmover",
     schema={
         # the maximum time to run, in seconds
-        Required("max-run-time"): int,
+        Optional("max-run-time"): int,
         # locale key, if this is a locale beetmover job
         Optional("locale"): str,
-        Optional("partner-public"): bool,
         Required("release-properties"): {
             "app-name": str,
             "app-version": str,
@@ -910,7 +951,6 @@ def build_beetmover_payload(config, task, task_def):
     release_properties = worker["release-properties"]
 
     task_def["payload"] = {
-        "maxRunTime": worker["max-run-time"],
         "releaseProperties": {
             "appName": release_properties["app-name"],
             "appVersion": release_properties["app-version"],
@@ -926,8 +966,6 @@ def build_beetmover_payload(config, task, task_def):
         task_def["payload"]["locale"] = worker["locale"]
     if worker.get("artifact-map"):
         task_def["payload"]["artifactMap"] = worker["artifact-map"]
-    if worker.get("partner-public"):
-        task_def["payload"]["is_partner_repack_public"] = worker["partner-public"]
     if release_config:
         task_def["payload"].update(release_config)
 
@@ -936,7 +974,7 @@ def build_beetmover_payload(config, task, task_def):
     "beetmover-push-to-release",
     schema={
         # the maximum time to run, in seconds
-        Required("max-run-time"): int,
+        Optional("max-run-time"): int,
         Required("product"): str,
     },
 )
@@ -946,7 +984,6 @@ def build_beetmover_push_to_release_payload(config, task, task_def):
     partners = [f"{p}/{s}" for p, s, _ in get_partners_to_be_published(config)]
 
     task_def["payload"] = {
-        "maxRunTime": worker["max-run-time"],
         "product": worker["product"],
         "version": release_config["version"],
         "build_number": release_config["build_number"],
@@ -957,7 +994,7 @@ def build_beetmover_push_to_release_payload(config, task, task_def):
 @payload_builder(
     "beetmover-import-from-gcs-to-artifact-registry",
     schema={
-        Required("max-run-time"): int,
+        Optional("max-run-time"): int,
         Required("gcs-sources"): [str],
         Required("product"): str,
     },
@@ -972,7 +1009,7 @@ def build_import_from_gcs_to_artifact_registry_payload(config, task, task_def):
 @payload_builder(
     "beetmover-maven",
     schema={
-        Required("max-run-time"): int,
+        Optional("max-run-time"): int,
         Required("release-properties"): {
             "app-name": str,
             "app-version": str,
@@ -1241,6 +1278,23 @@ def build_push_msix_payload(config, task, task_def):
         task_def["payload"]["publishMode"] = worker["publish-mode"]
 
 
+@payload_builder(
+    "shipit-update-product-channel-version",
+    schema={
+        Required("product"): str,
+        Required("channel"): str,
+        Required("version"): str,
+    },
+)
+def build_ship_it_update_product_channel_version_payload(config, task, task_def):
+    worker = task["worker"]
+    task_def["payload"] = {
+        "product": worker["product"],
+        "version": worker["version"],
+        "channel": worker["channel"],
+    }
+
+
 @payload_builder(
     "shipit-shipped",
     schema={
@@ -1327,6 +1381,23 @@ def build_push_addons_payload(config, task, task_def):
             }
         ],
         Optional("merge-info"): object,
+        Optional("android-l10n-import-info"): {
+            Required("from-repo-url"): str,
+            Required("toml-info"): [
+                {
+                    Required("toml-path"): str,
+                    Required("dest-path"): str,
+                }
+            ],
+        },
+        Optional("android-l10n-sync-info"): {
+            Required("from-repo-url"): str,
+            Required("toml-info"): [
+                {
+                    Required("toml-path"): str,
+                }
+            ],
+        },
     },
 )
 def build_treescript_payload(config, task, task_def):
@@ -1405,6 +1476,38 @@ def build_treescript_payload(config, task, task_def):
         task_def["payload"]["merge_info"] = merge_info
         actions.append("merge_day")
 
+    if worker.get("android-l10n-import-info"):
+        android_l10n_import_info = {}
+        for k, v in worker["android-l10n-import-info"].items():
+            android_l10n_import_info[k.replace("-", "_")] = worker[
+                "android-l10n-import-info"
+            ][k]
+        android_l10n_import_info["toml_info"] = [
+            {
+                param_name.replace("-", "_"): param_value
+                for param_name, param_value in entry.items()
+            }
+            for entry in worker["android-l10n-import-info"]["toml-info"]
+        ]
+        task_def["payload"]["android_l10n_import_info"] = android_l10n_import_info
+        actions.append("android_l10n_import")
+
+    if worker.get("android-l10n-sync-info"):
+        android_l10n_sync_info = {}
+        for k, v in worker["android-l10n-sync-info"].items():
+            android_l10n_sync_info[k.replace("-", "_")] = worker[
+                "android-l10n-sync-info"
+            ][k]
+        android_l10n_sync_info["toml_info"] = [
+            {
+                param_name.replace("-", "_"): param_value
+                for param_name, param_value in entry.items()
+            }
+            for entry in worker["android-l10n-sync-info"]["toml-info"]
+        ]
+        task_def["payload"]["android_l10n_sync_info"] = android_l10n_sync_info
+        actions.append("android_l10n_sync")
+
     if worker["push"]:
         actions.append("push")
 
@@ -1433,6 +1536,23 @@ def build_treescript_payload(config, task, task_def):
         Optional("dontbuild"): bool,
         Optional("tags"): [Any("buildN", "release", None)],
         Optional("force-dry-run"): bool,
+        Optional("android-l10n-import-info"): {
+            Required("from-repo-url"): str,
+            Required("toml-info"): [
+                {
+                    Required("toml-path"): str,
+                    Required("dest-path"): str,
+                }
+            ],
+        },
+        Optional("android-l10n-sync-info"): {
+            Required("from-branch"): str,
+            Required("toml-info"): [
+                {
+                    Required("toml-path"): str,
+                }
+            ],
+        },
         Optional("l10n-bump-info"): [
             {
                 Required("name"): str,
@@ -1468,6 +1588,38 @@ def build_landoscript_payload(config, task, task_def):
     if worker.get("force-dry-run"):
         task_def["payload"]["dry_run"] = True
 
+    if worker.get("android-l10n-import-info"):
+        android_l10n_import_info = {}
+        for k, v in worker["android-l10n-import-info"].items():
+            android_l10n_import_info[k.replace("-", "_")] = worker[
+                "android-l10n-import-info"
+            ][k]
+        android_l10n_import_info["toml_info"] = [
+            {
+                param_name.replace("-", "_"): param_value
+                for param_name, param_value in entry.items()
+            }
+            for entry in worker["android-l10n-import-info"]["toml-info"]
+        ]
+        task_def["payload"]["android_l10n_import_info"] = android_l10n_import_info
+        actions.append("android_l10n_import")
+
+    if worker.get("android-l10n-sync-info"):
+        android_l10n_sync_info = {}
+        for k, v in worker["android-l10n-sync-info"].items():
+            android_l10n_sync_info[k.replace("-", "_")] = worker[
+                "android-l10n-sync-info"
+            ][k]
+        android_l10n_sync_info["toml_info"] = [
+            {
+                param_name.replace("-", "_"): param_value
+                for param_name, param_value in entry.items()
+            }
+            for entry in worker["android-l10n-sync-info"]["toml-info"]
+        ]
+        task_def["payload"]["android_l10n_sync_info"] = android_l10n_sync_info
+        actions.append("android_l10n_sync")
+
     if worker.get("l10n-bump-info"):
         l10n_bump_info = []
         l10n_repo_urls = set()
@@ -1578,27 +1730,46 @@ def set_implementation(config, tasks):
     Set the worker implementation based on the worker-type alias.
     """
     for task in tasks:
-        if "implementation" in task["worker"]:
-            yield task
-            continue
-
-        impl, os = worker_type_implementation(
+        default_worker_implementation, default_os = worker_type_implementation(
             config.graph_config, config.params, task["worker-type"]
         )
 
+        worker = task.setdefault("worker", {})
         tags = task.setdefault("tags", {})
-        tags["worker-implementation"] = impl
+
+        worker_implementation = worker.get(
+            "implementation", default_worker_implementation
+        )
+        tag_worker_implementation = _get_worker_implementation_tag(
+            config, task["worker-type"], worker_implementation
+        )
+        if worker_implementation:
+            worker["implementation"] = worker_implementation
+            tags["worker-implementation"] = tag_worker_implementation
+
+        os = worker.get("os", default_os)
         if os:
             tags["os"] = os
-
-        worker = task.setdefault("worker", {})
-        worker["implementation"] = impl
-        if os:
             worker["os"] = os
 
         yield task
 
 
+def _get_worker_implementation_tag(config, task_worker_type, worker_implementation):
+    # Scriptworkers have different types of payload and each sets its own
+    # worker-implementation. Per bug 1955941, we want to bundle them all in one category
+    # through their tags.
+    provisioner_id, _ = get_worker_type(
+        config.graph_config,
+        config.params,
+        task_worker_type,
+    )
+    if provisioner_id in ("scriptworker-k8s", "scriptworker-prov-v1"):
+        return "scriptworker"
+
+    return worker_implementation
+
+
 @transforms.add
 def set_defaults(config, tasks):
     for task in tasks:
@@ -1625,20 +1796,15 @@ def set_defaults(config, tasks):
         elif worker["implementation"] == "generic-worker":
             worker.setdefault("env", {})
             worker.setdefault("os-groups", [])
-            if worker["os-groups"] and worker["os"] != "windows":
+            if worker["os-groups"] and worker["os"] not in (
+                "windows",
+                "linux",
+            ):
                 raise Exception(
                     "os-groups feature of generic-worker is only supported on "
-                    "Windows, not on {}".format(worker["os"])
+                    "Windows and Linux, not on {}".format(worker["os"])
                 )
             worker.setdefault("chain-of-trust", False)
-        elif worker["implementation"] in (
-            "scriptworker-signing",
-            "beetmover",
-            "beetmover-push-to-release",
-            "beetmover-maven",
-            "beetmover-import-from-gcs-to-artifact-registry",
-        ):
-            worker.setdefault("max-run-time", 600)
         elif worker["implementation"] == "push-apk":
             worker.setdefault("commit", False)
 
@@ -1672,13 +1838,13 @@ def task_name_from_label(config, tasks):
         if "label" not in task:
             if taskname is None:
                 raise Exception("task has neither a name nor a label")
-            task["label"] = "{}-{}".format(config.kind, taskname)
+            task["label"] = f"{config.kind}-{taskname}"
         yield task
 
 
 UNSUPPORTED_SHIPPING_PRODUCT_ERROR = """\
 The shipping product {product} is not in the list of configured products in
-`taskcluster/ci/config.yml'.
+`taskcluster/config.yml'.
 """
 
 
@@ -1723,11 +1889,19 @@ def add_generic_index_routes(config, task):
     subs["product"] = index["product"]
     subs["trust-domain"] = config.graph_config["trust-domain"]
     subs["branch_rev"] = get_branch_rev(config)
+    try:
+        subs["branch_git_rev"] = get_branch_git_rev(config)
+    except KeyError:
+        pass
 
     project = config.params.get("project")
 
     for tpl in V2_ROUTE_TEMPLATES:
-        routes.append(tpl.format(**subs))
+        try:
+            routes.append(tpl.format(**subs))
+        except KeyError:
+            # Ignore errors that arise from branch_git_rev not being set.
+            pass
 
     # Additionally alias all tasks for "trunk" repos into a common
     # namespace.
@@ -1756,9 +1930,17 @@ def add_shippable_index_routes(config, task):
     subs["product"] = index["product"]
     subs["trust-domain"] = config.graph_config["trust-domain"]
     subs["branch_rev"] = get_branch_rev(config)
+    try:
+        subs["branch_git_rev"] = get_branch_git_rev(config)
+    except KeyError:
+        pass
 
     for tpl in V2_SHIPPABLE_TEMPLATES:
-        routes.append(tpl.format(**subs))
+        try:
+            routes.append(tpl.format(**subs))
+        except KeyError:
+            # Ignore errors that arise from branch_git_rev not being set.
+            pass
 
     # Also add routes for en-US
     task = add_shippable_l10n_index_routes(config, task, force_locale="en-US")
@@ -1902,10 +2084,11 @@ def add_index_routes(config, tasks):
         rank = index.get("rank", "by-tier")
 
         if rank == "by-tier":
-            # rank is zero for non-tier-1 tasks and based on pushid for others;
-            # this sorts tier-{2,3} builds below tier-1 in the index
+            # rank is one for non-tier-1 tasks and based on pushid for others;
+            # this sorts tier-{2,3} builds below tier-1 in the index, but above
+            # eager-index
             tier = task.get("treeherder", {}).get("tier", 3)
-            extra_index["rank"] = 0 if tier > 1 else int(config.params["build_date"])
+            extra_index["rank"] = 1 if tier > 1 else int(config.params["build_date"])
         elif rank == "build_date":
             extra_index["rank"] = int(config.params["build_date"])
         else:
@@ -2012,6 +2195,43 @@ def set_task_and_artifact_expiry(config, jobs):
         yield job
 
 
+def group_name_variant(group_names, groupSymbol):
+    # iterate through variants, allow for Base-[variant_list]
+    # sorting longest->shortest allows for finding variants when
+    # other variants have a suffix that is a subset
+    variant_symbols = sorted(
+        [
+            (
+                v,
+                TEST_VARIANTS[v]["suffix"],
+                TEST_VARIANTS[v].get("description", "{description}"),
+            )
+            for v in TEST_VARIANTS
+            if TEST_VARIANTS[v].get("suffix", "")
+        ],
+        key=lambda tup: len(tup[1]),
+        reverse=True,
+    )
+
+    # strip known variants
+    # build a list of known variants
+    base_symbol = groupSymbol
+    found_variants = []
+    for variant, suffix, description in variant_symbols:
+        if f"-{suffix}" in base_symbol:
+            base_symbol = base_symbol.replace(f"-{suffix}", "")
+            found_variants.append((variant, description))
+
+    if base_symbol not in group_names:
+        return ""
+
+    description = group_names[base_symbol]
+    for variant, desc in found_variants:
+        description = desc.format(description=description)
+
+    return description
+
+
 @transforms.add
 def build_task(config, tasks):
     for task in tasks:
@@ -2052,10 +2272,13 @@ def build_task(config, tasks):
             groupSymbol, symbol = split_symbol(task_th["symbol"])
             if groupSymbol != "?":
                 treeherder["groupSymbol"] = groupSymbol
-                if groupSymbol not in group_names:
-                    path = os.path.join(config.path, task.get("job-from", ""))
+                description = group_names.get(
+                    groupSymbol, group_name_variant(group_names, groupSymbol)
+                )
+                if not description:
+                    path = os.path.join(config.path, task.get("task-from", ""))
                     raise Exception(UNKNOWN_GROUP_NAME.format(groupSymbol, path))
-                treeherder["groupName"] = group_names[groupSymbol]
+                treeherder["groupName"] = description
             treeherder["symbol"] = symbol
             if len(symbol) > 25 or len(groupSymbol) > 25:
                 raise RuntimeError(
@@ -2095,6 +2318,8 @@ def build_task(config, tasks):
                 "kind": config.kind,
                 "label": task["label"],
                 "retrigger": "true" if attributes.get("retrigger", False) else "false",
+                "project": config.params["project"],
+                "trust-domain": config.graph_config["trust-domain"],
             }
         )
 
@@ -2174,7 +2399,7 @@ def build_task(config, tasks):
             )
         attributes.setdefault("shipping_product", task["shipping-product"])
 
-        # Set MOZ_AUTOMATION on all jobs.
+        # Set some MOZ_* settings  on all jobs.
         if task["worker"]["implementation"] in (
             "generic-worker",
             "docker-worker",
@@ -2182,7 +2407,15 @@ def build_task(config, tasks):
             payload = task_def.get("payload")
             if payload:
                 env = payload.setdefault("env", {})
-                env["MOZ_AUTOMATION"] = "1"
+                env.update(
+                    {
+                        "MOZ_AUTOMATION": "1",
+                        "MOZ_BUILD_DATE": config.params["moz_build_date"],
+                        "MOZ_SCM_LEVEL": config.params["level"],
+                        "MOZ_SOURCE_CHANGESET": get_branch_rev(config),
+                        "MOZ_SOURCE_REPO": get_branch_repo(config),
+                    }
+                )
 
         dependencies = task.get("dependencies", {})
         if_dependencies = task.get("if-dependencies", [])
