@@ -8,14 +8,11 @@ set -v
 
 cd $GECKO_PATH
 
-# Needed for Nexus
-export PATH=$MOZ_FETCHES_DIR/jdk-8/bin:$PATH
-
 . taskcluster/scripts/misc/android-gradle-dependencies/before.sh
 
 export MOZCONFIG=mobile/android/config/mozconfigs/android-arm-gradle-dependencies/nightly-lite
 ./mach build
-./mach gradle downloadDependencies
-./mach android gradle-dependencies
+./mach gradle downloadDependencies --no-configuration-cache
+./mach android gradle-dependencies --no-configuration-cache
 
 . taskcluster/scripts/misc/android-gradle-dependencies/after.sh
