@@ -48,7 +48,12 @@ PARAMS_NEW_CONFIG.update(
                 t.kind == "test"
                 and t.attributes["unittest_suite"] == "mochitest-browser-chrome"
                 and t.attributes["test_platform"] == "linux1804-64-qr/opt"
-                and ("spi-nw" not in t.label and "a11y-checks" not in t.label)
+                and (
+                    "spi-nw" not in t.label
+                    and "a11y-checks" not in t.label
+                    and "vt" not in t.label
+                    and "ioi" not in t.label
+                )
             ),
             32,
             id="mochitest-browser-chrome",
@@ -74,7 +79,12 @@ def test_tasks_new_config_false(full_task_graph, filter_tasks, func, min_expecte
                 t.kind == "test"
                 and t.attributes["unittest_suite"] == "mochitest-browser-chrome"
                 and t.attributes["test_platform"] == "linux1804-64-qr/opt"
-                and ("spi-nw" not in t.label and "a11y-checks" not in t.label)
+                and (
+                    "spi-nw" not in t.label
+                    and "a11y-checks" not in t.label
+                    and "vt" not in t.label
+                    and "ioi" not in t.label
+                )
             ),
             32,
             id="mochitest-browser-chrome",
